@@ -20,13 +20,16 @@ package ap
 import (
 	"net/url"
 
-	"github.com/superseriousbusiness/activity/streams/vocab"
+	"codeberg.org/superseriousbusiness/activity/streams/vocab"
 )
 
 // IsActivityable returns whether AS vocab type name is acceptable as Activityable.
 func IsActivityable(typeName string) bool {
-	return isActivity(typeName) ||
-		isIntransitiveActivity(typeName)
+	return isActivity(typeName)
+	// See interfaces_test.go comment
+	// about intransitive activities:
+	//
+	// || isIntransitiveActivity(typeName)
 }
 
 // ToActivityable safely tries to cast vocab.Type as Activityable, also checking for expected AS type names.
@@ -73,7 +76,8 @@ func IsStatusable(typeName string) bool {
 		ObjectEvent,
 		ObjectPlace,
 		ObjectProfile,
-		ActivityQuestion:
+		ActivityQuestion,
+		ObjectAlbum:
 		return true
 	default:
 		return false
@@ -125,16 +129,13 @@ func ToPollOptionable(t vocab.Type) (PollOptionable, bool) {
 }
 
 // IsAccept returns whether AS vocab type name
-// is something that can be cast to Accept.
+// is something that can be cast to Acceptable.
 func IsAcceptable(typeName string) bool {
 	return typeName == ActivityAccept
 }
 
-// ToAcceptable safely tries to cast vocab.Type as vocab.ActivityStreamsAccept.
-//
-// TODO: Add additional "Accept" types here, eg., "ApproveReply" from
-// https://codeberg.org/fediverse/fep/src/branch/main/fep/5624/fep-5624.md
-func ToAcceptable(t vocab.Type) (vocab.ActivityStreamsAccept, bool) {
+// ToAcceptable safely tries to cast vocab.Type as Acceptable.
+func ToAcceptable(t vocab.Type) (Acceptable, bool) {
 	acceptable, ok := t.(vocab.ActivityStreamsAccept)
 	if !ok || !IsAcceptable(t.GetTypeName()) {
 		return nil, false
@@ -142,6 +143,28 @@ func ToAcceptable(t vocab.Type) (vocab.ActivityStreamsAccept, bool) {
 	return acceptable, true
 }
 
+// IsApprovable returns whether AS vocab type name
+// is something that can be cast to Approvable.
+func IsApprovable(typeName string) bool {
+	switch typeName {
+	case ObjectLikeApproval,
+		ObjectReplyApproval,
+		ObjectAnnounceApproval:
+		return true
+	default:
+		return false
+	}
+}
+
+// ToAcceptable safely tries to cast vocab.Type as Approvable.
+func ToApprovable(t vocab.Type) (Approvable, bool) {
+	approvable, ok := t.(Approvable)
+	if !ok || !IsApprovable(t.GetTypeName()) {
+		return nil, false
+	}
+	return approvable, true
+}
+
 // Activityable represents the minimum activitypub interface for representing an 'activity'.
 // (see: IsActivityable() for types implementing this, though you MUST make sure to check
 // the typeName as this bare interface may be implementable by non-Activityable types).
@@ -184,6 +207,8 @@ type Accountable interface {
 	WithEndpoints
 	WithTag
 	WithPublished
+	WithUpdated
+	WithImage
 }
 
 // Statusable represents the minimum activitypub interface for representing a 'status'.
@@ -196,16 +221,19 @@ type Statusable interface {
 	WithName
 	WithInReplyTo
 	WithPublished
+	WithUpdated
 	WithURL
 	WithAttributedTo
 	WithTo
 	WithCc
 	WithSensitive
-	WithConversation
 	WithContent
 	WithAttachment
 	WithTag
 	WithReplies
+}
+
+type InteractionPolicyAware interface {
 	WithInteractionPolicy
 	WithApprovedBy
 }
@@ -241,6 +269,19 @@ type PollOptionable interface {
 // interface for representing an Accept.
 type Acceptable interface {
 	Activityable
+
+	WithTarget
+	WithResult
+}
+
+// Approvable represents the minimum activitypub interface
+// for a LikeApproval, ReplyApproval, or AnnounceApproval.
+type Approvable interface {
+	vocab.Type
+
+	WithAttributedTo
+	WithObject
+	WithTarget
 }
 
 // Attachmentable represents the minimum activitypub interface for representing a 'mediaAttachment'. (see: IsAttachmentable).
@@ -434,6 +475,7 @@ type WithValue interface {
 // WithImage represents an activity with ActivityStreamsImageProperty
 type WithImage interface {
 	GetActivityStreamsImage() vocab.ActivityStreamsImageProperty
+	SetActivityStreamsImage(vocab.ActivityStreamsImageProperty)
 }
 
 // WithSummary represents an activity with ActivityStreamsSummaryProperty
@@ -550,10 +592,6 @@ type WithSensitive interface {
 	SetActivityStreamsSensitive(vocab.ActivityStreamsSensitiveProperty)
 }
 
-// WithConversation ...
-type WithConversation interface { // TODO
-}
-
 // WithContent represents an activity with ActivityStreamsContentProperty
 type WithContent interface {
 	GetActivityStreamsContent() vocab.ActivityStreamsContentProperty
@@ -701,3 +739,9 @@ type WithApprovedBy interface {
 	GetGoToSocialApprovedBy() vocab.GoToSocialApprovedByProperty
 	SetGoToSocialApprovedBy(vocab.GoToSocialApprovedByProperty)
 }
+
+// WithVotersCount represents an activity or object the result property.
+type WithResult interface {
+	GetActivityStreamsResult() vocab.ActivityStreamsResultProperty
+	SetActivityStreamsResult(vocab.ActivityStreamsResultProperty)
+}
