@@ -22,8 +22,8 @@ import (
 	"net/url"
 	"time"
 
-	"github.com/superseriousbusiness/activity/streams"
-	"github.com/superseriousbusiness/activity/streams/vocab"
+	"codeberg.org/superseriousbusiness/activity/streams"
+	"codeberg.org/superseriousbusiness/activity/streams/vocab"
 	"github.com/superseriousbusiness/gotosocial/internal/gtserror"
 )
 
@@ -408,6 +408,25 @@ func SetPublished(with WithPublished, published time.Time) {
 	publishProp.Set(published)
 }
 
+// GetUpdated returns the time contained in the Updated property of 'with'.
+func GetUpdated(with WithUpdated) time.Time {
+	updateProp := with.GetActivityStreamsUpdated()
+	if updateProp == nil || !updateProp.IsXMLSchemaDateTime() {
+		return time.Time{}
+	}
+	return updateProp.Get()
+}
+
+// SetUpdated sets the given time on the Updated property of 'with'.
+func SetUpdated(with WithUpdated, updated time.Time) {
+	updateProp := with.GetActivityStreamsUpdated()
+	if updateProp == nil {
+		updateProp = streams.NewActivityStreamsUpdatedProperty()
+		with.SetActivityStreamsUpdated(updateProp)
+	}
+	updateProp.Set(updated)
+}
+
 // GetEndTime returns the time contained in the EndTime property of 'with'.
 func GetEndTime(with WithEndTime) time.Time {
 	endTimeProp := with.GetActivityStreamsEndTime()
@@ -501,11 +520,11 @@ func SetDiscoverable(with WithDiscoverable, discoverable bool) {
 
 // GetManuallyApprovesFollowers returns the boolean contained in the ManuallyApprovesFollowers property of 'with'.
 //
-// Returns default 'true' if property unusable or not set.
+// Returns default 'false' if property unusable or not set.
 func GetManuallyApprovesFollowers(with WithManuallyApprovesFollowers) bool {
 	mafProp := with.GetActivityStreamsManuallyApprovesFollowers()
 	if mafProp == nil || !mafProp.IsXMLSchemaBoolean() {
-		return true
+		return false
 	}
 	return mafProp.Get()
 }
