@@ -96,10 +96,11 @@ func (suite *AccountsGetTestSuite) TestAccountsGetFromTop() {
       "avatar_static": "",
       "header": "http://localhost:8080/assets/default_header.webp",
       "header_static": "http://localhost:8080/assets/default_header.webp",
+      "header_description": "Flat gray background (default header).",
       "followers_count": 1,
       "following_count": 1,
-      "statuses_count": 8,
-      "last_status_at": "2021-07-28",
+      "statuses_count": 9,
+      "last_status_at": "2024-11-01",
       "emojis": [],
       "fields": [
         {
@@ -113,7 +114,8 @@ func (suite *AccountsGetTestSuite) TestAccountsGetFromTop() {
           "verified_at": null
         }
       ],
-      "hide_collections": true
+      "hide_collections": true,
+      "group": false
     },
     "created_by_application_id": "01F8MGY43H3N2C8EWPR2FPYEXG"
   },
@@ -154,6 +156,7 @@ func (suite *AccountsGetTestSuite) TestAccountsGetFromTop() {
       "avatar_static": "",
       "header": "http://localhost:8080/assets/default_header.webp",
       "header_static": "http://localhost:8080/assets/default_header.webp",
+      "header_description": "Flat gray background (default header).",
       "followers_count": 1,
       "following_count": 1,
       "statuses_count": 4,
@@ -167,7 +170,8 @@ func (suite *AccountsGetTestSuite) TestAccountsGetFromTop() {
           "name": "admin",
           "color": ""
         }
-      ]
+      ],
+      "group": false
     },
     "created_by_application_id": "01F8MGXQRHYF5QPMTMXP78QC2F"
   },
@@ -200,7 +204,7 @@ func (suite *AccountsGetTestSuite) TestAccountsGetFromTop() {
       "display_name": "",
       "locked": false,
       "discoverable": true,
-      "bot": false,
+      "bot": true,
       "created_at": "2020-05-17T13:10:59.000Z",
       "note": "",
       "url": "http://localhost:8080/@localhost:8080",
@@ -208,14 +212,79 @@ func (suite *AccountsGetTestSuite) TestAccountsGetFromTop() {
       "avatar_static": "",
       "header": "http://localhost:8080/assets/default_header.webp",
       "header_static": "http://localhost:8080/assets/default_header.webp",
+      "header_description": "Flat gray background (default header).",
       "followers_count": 0,
       "following_count": 0,
       "statuses_count": 0,
       "last_status_at": null,
       "emojis": [],
-      "fields": []
+      "fields": [],
+      "group": false
     }
   },
+  {
+    "id": "01JPCMD83Y4WR901094YES3QC5",
+    "username": "media_mogul",
+    "domain": null,
+    "created_at": "2025-03-15T11:08:00.000Z",
+    "email": "media.mogul@example.org",
+    "ip": null,
+    "ips": [],
+    "locale": "en",
+    "invite_request": null,
+    "role": {
+      "id": "user",
+      "name": "user",
+      "color": "",
+      "permissions": "0",
+      "highlighted": false
+    },
+    "confirmed": true,
+    "approved": true,
+    "disabled": false,
+    "silenced": false,
+    "suspended": false,
+    "account": {
+      "id": "01JPCMD83Y4WR901094YES3QC5",
+      "username": "media_mogul",
+      "acct": "media_mogul",
+      "display_name": "",
+      "locked": false,
+      "discoverable": false,
+      "bot": false,
+      "created_at": "2025-03-15T11:08:00.000Z",
+      "note": "<p>I'm a test account that posts a shitload of media and I have my account rendered in \"gallery\" mode</p>",
+      "url": "http://localhost:8080/@media_mogul",
+      "avatar": "http://localhost:8080/fileserver/01JPCMD83Y4WR901094YES3QC5/avatar/original/01JPHQZ0ZHC2AXJK1JQNXRXQZN.jpeg",
+      "avatar_static": "http://localhost:8080/fileserver/01JPCMD83Y4WR901094YES3QC5/avatar/small/01JPHQZ0ZHC2AXJK1JQNXRXQZN.jpeg",
+      "avatar_description": "DESCRIPTION_GOES_HERE",
+      "avatar_media_id": "01JPHQZ0ZHC2AXJK1JQNXRXQZN",
+      "header": "http://localhost:8080/fileserver/01JPCMD83Y4WR901094YES3QC5/header/original/01JPHRB7F2RXPTEQFRYC85EPD9.png",
+      "header_static": "http://localhost:8080/fileserver/01JPCMD83Y4WR901094YES3QC5/header/small/01JPHRB7F2RXPTEQFRYC85EPD9.webp",
+      "header_description": "DESCRIPTION_GOES_HERE",
+      "header_media_id": "01JPHRB7F2RXPTEQFRYC85EPD9",
+      "followers_count": 0,
+      "following_count": 0,
+      "statuses_count": 2,
+      "last_status_at": "2025-03-15",
+      "emojis": [],
+      "fields": [
+        {
+          "name": "I'm going to post a lot of",
+          "value": "media!",
+          "verified_at": null
+        },
+        {
+          "name": "and there's nothing",
+          "value": "you can do about it",
+          "verified_at": null
+        }
+      ],
+      "enable_rss": true,
+      "group": false
+    },
+    "created_by_application_id": "01HT5P2YHDMPAAD500NDAY8JW1"
+  },
   {
     "id": "01F8MH1H7YV1Z7D2C8K2730QBF",
     "username": "the_mighty_zork",
@@ -252,16 +321,19 @@ func (suite *AccountsGetTestSuite) TestAccountsGetFromTop() {
       "avatar": "http://localhost:8080/fileserver/01F8MH1H7YV1Z7D2C8K2730QBF/avatar/original/01F8MH58A357CV5K7R7TJMSH6S.jpg",
       "avatar_static": "http://localhost:8080/fileserver/01F8MH1H7YV1Z7D2C8K2730QBF/avatar/small/01F8MH58A357CV5K7R7TJMSH6S.webp",
       "avatar_description": "a green goblin looking nasty",
+      "avatar_media_id": "01F8MH58A357CV5K7R7TJMSH6S",
       "header": "http://localhost:8080/fileserver/01F8MH1H7YV1Z7D2C8K2730QBF/header/original/01PFPMWK2FF0D9WMHEJHR07C3Q.jpg",
       "header_static": "http://localhost:8080/fileserver/01F8MH1H7YV1Z7D2C8K2730QBF/header/small/01PFPMWK2FF0D9WMHEJHR07C3Q.webp",
       "header_description": "A very old-school screenshot of the original team fortress mod for quake",
+      "header_media_id": "01PFPMWK2FF0D9WMHEJHR07C3Q",
       "followers_count": 2,
       "following_count": 2,
-      "statuses_count": 8,
-      "last_status_at": "2024-01-10",
+      "statuses_count": 9,
+      "last_status_at": "2024-11-01",
       "emojis": [],
       "fields": [],
-      "enable_rss": true
+      "enable_rss": true,
+      "group": false
     },
     "created_by_application_id": "01F8MGY43H3N2C8EWPR2FPYEXG"
   },
@@ -302,12 +374,14 @@ func (suite *AccountsGetTestSuite) TestAccountsGetFromTop() {
       "avatar_static": "",
       "header": "http://localhost:8080/assets/default_header.webp",
       "header_static": "http://localhost:8080/assets/default_header.webp",
+      "header_description": "Flat gray background (default header).",
       "followers_count": 0,
       "following_count": 0,
       "statuses_count": 0,
       "last_status_at": null,
       "emojis": [],
-      "fields": []
+      "fields": [],
+      "group": false
     },
     "created_by_application_id": "01F8MGY43H3N2C8EWPR2FPYEXG"
   },
@@ -348,12 +422,14 @@ func (suite *AccountsGetTestSuite) TestAccountsGetFromTop() {
       "avatar_static": "",
       "header": "http://localhost:8080/assets/default_header.webp",
       "header_static": "http://localhost:8080/assets/default_header.webp",
+      "header_description": "Flat gray background (default header).",
       "followers_count": 0,
       "following_count": 0,
       "statuses_count": 1,
       "last_status_at": "2023-11-02",
       "emojis": [],
-      "fields": []
+      "fields": [],
+      "group": false
     }
   },
   {
@@ -393,12 +469,14 @@ func (suite *AccountsGetTestSuite) TestAccountsGetFromTop() {
       "avatar_static": "",
       "header": "http://localhost:8080/assets/default_header.webp",
       "header_static": "http://localhost:8080/assets/default_header.webp",
+      "header_description": "Flat gray background (default header).",
       "followers_count": 0,
       "following_count": 0,
-      "statuses_count": 3,
-      "last_status_at": "2021-09-11",
+      "statuses_count": 4,
+      "last_status_at": "2024-11-01",
       "emojis": [],
-      "fields": []
+      "fields": [],
+      "group": false
     }
   },
   {
@@ -439,12 +517,14 @@ func (suite *AccountsGetTestSuite) TestAccountsGetFromTop() {
       "header": "http://localhost:8080/fileserver/062G5WYKY35KKD12EMSM3F8PJ8/header/original/01PFPMWK2FF0D9WMHEJHR07C3R.jpg",
       "header_static": "http://localhost:8080/fileserver/062G5WYKY35KKD12EMSM3F8PJ8/header/small/01PFPMWK2FF0D9WMHEJHR07C3R.webp",
       "header_description": "tweet from thoughts of dog: i drank. all the water. in my bowl. earlier. but just now. i returned. to the same bowl. and it was. full again.. the bowl. is haunted",
+      "header_media_id": "01PFPMWK2FF0D9WMHEJHR07C3R",
       "followers_count": 0,
       "following_count": 0,
       "statuses_count": 0,
       "last_status_at": null,
       "emojis": [],
-      "fields": []
+      "fields": [],
+      "group": false
     }
   },
   {
@@ -484,12 +564,14 @@ func (suite *AccountsGetTestSuite) TestAccountsGetFromTop() {
       "avatar_static": "",
       "header": "http://localhost:8080/assets/default_header.webp",
       "header_static": "http://localhost:8080/assets/default_header.webp",
+      "header_description": "Flat gray background (default header).",
       "followers_count": 0,
       "following_count": 0,
       "statuses_count": 0,
       "last_status_at": null,
       "emojis": [],
-      "fields": []
+      "fields": [],
+      "group": false
     }
   }
 ]`, dst.String())
@@ -528,18 +610,18 @@ func (suite *AccountsGetTestSuite) TestAccountsMinID() {
 	}
 
 	link := recorder.Header().Get("Link")
-	suite.Equal(`<http://localhost:8080/api/v2/admin/accounts?limit=1&max_id=%2F%40localhost%3A8080>; rel="next", <http://localhost:8080/api/v2/admin/accounts?limit=1&min_id=%2F%40localhost%3A8080>; rel="prev"`, link)
+	suite.Equal(`<http://localhost:8080/api/v2/admin/accounts?limit=1&max_id=%2F%40media_mogul>; rel="next", <http://localhost:8080/api/v2/admin/accounts?limit=1&min_id=%2F%40media_mogul>; rel="prev"`, link)
 
 	suite.Equal(`[
   {
-    "id": "01AY6P665V14JJR0AFVRT7311Y",
-    "username": "localhost:8080",
+    "id": "01JPCMD83Y4WR901094YES3QC5",
+    "username": "media_mogul",
     "domain": null,
-    "created_at": "2020-05-17T13:10:59.000Z",
-    "email": "",
+    "created_at": "2025-03-15T11:08:00.000Z",
+    "email": "media.mogul@example.org",
     "ip": null,
     "ips": [],
-    "locale": "",
+    "locale": "en",
     "invite_request": null,
     "role": {
       "id": "user",
@@ -548,33 +630,51 @@ func (suite *AccountsGetTestSuite) TestAccountsMinID() {
       "permissions": "0",
       "highlighted": false
     },
-    "confirmed": false,
-    "approved": false,
+    "confirmed": true,
+    "approved": true,
     "disabled": false,
     "silenced": false,
     "suspended": false,
     "account": {
-      "id": "01AY6P665V14JJR0AFVRT7311Y",
-      "username": "localhost:8080",
-      "acct": "localhost:8080",
+      "id": "01JPCMD83Y4WR901094YES3QC5",
+      "username": "media_mogul",
+      "acct": "media_mogul",
       "display_name": "",
       "locked": false,
-      "discoverable": true,
+      "discoverable": false,
       "bot": false,
-      "created_at": "2020-05-17T13:10:59.000Z",
-      "note": "",
-      "url": "http://localhost:8080/@localhost:8080",
-      "avatar": "",
-      "avatar_static": "",
-      "header": "http://localhost:8080/assets/default_header.webp",
-      "header_static": "http://localhost:8080/assets/default_header.webp",
+      "created_at": "2025-03-15T11:08:00.000Z",
+      "note": "<p>I'm a test account that posts a shitload of media and I have my account rendered in \"gallery\" mode</p>",
+      "url": "http://localhost:8080/@media_mogul",
+      "avatar": "http://localhost:8080/fileserver/01JPCMD83Y4WR901094YES3QC5/avatar/original/01JPHQZ0ZHC2AXJK1JQNXRXQZN.jpeg",
+      "avatar_static": "http://localhost:8080/fileserver/01JPCMD83Y4WR901094YES3QC5/avatar/small/01JPHQZ0ZHC2AXJK1JQNXRXQZN.jpeg",
+      "avatar_description": "DESCRIPTION_GOES_HERE",
+      "avatar_media_id": "01JPHQZ0ZHC2AXJK1JQNXRXQZN",
+      "header": "http://localhost:8080/fileserver/01JPCMD83Y4WR901094YES3QC5/header/original/01JPHRB7F2RXPTEQFRYC85EPD9.png",
+      "header_static": "http://localhost:8080/fileserver/01JPCMD83Y4WR901094YES3QC5/header/small/01JPHRB7F2RXPTEQFRYC85EPD9.webp",
+      "header_description": "DESCRIPTION_GOES_HERE",
+      "header_media_id": "01JPHRB7F2RXPTEQFRYC85EPD9",
       "followers_count": 0,
       "following_count": 0,
-      "statuses_count": 0,
-      "last_status_at": null,
+      "statuses_count": 2,
+      "last_status_at": "2025-03-15",
       "emojis": [],
-      "fields": []
-    }
+      "fields": [
+        {
+          "name": "I'm going to post a lot of",
+          "value": "media!",
+          "verified_at": null
+        },
+        {
+          "name": "and there's nothing",
+          "value": "you can do about it",
+          "verified_at": null
+        }
+      ],
+      "enable_rss": true,
+      "group": false
+    },
+    "created_by_application_id": "01HT5P2YHDMPAAD500NDAY8JW1"
   }
 ]`, dst.String())
 }
