@@ -183,12 +183,14 @@ func (suite *ReportsGetTestSuite) TestReportsGetAll() {
         "avatar_static": "",
         "header": "http://localhost:8080/assets/default_header.webp",
         "header_static": "http://localhost:8080/assets/default_header.webp",
+        "header_description": "Flat gray background (default header).",
         "followers_count": 0,
         "following_count": 0,
-        "statuses_count": 3,
-        "last_status_at": "2021-09-11",
+        "statuses_count": 4,
+        "last_status_at": "2024-11-01",
         "emojis": [],
-        "fields": []
+        "fields": [],
+        "group": false
       }
     },
     "target_account": {
@@ -228,10 +230,11 @@ func (suite *ReportsGetTestSuite) TestReportsGetAll() {
         "avatar_static": "",
         "header": "http://localhost:8080/assets/default_header.webp",
         "header_static": "http://localhost:8080/assets/default_header.webp",
+        "header_description": "Flat gray background (default header).",
         "followers_count": 1,
         "following_count": 1,
-        "statuses_count": 8,
-        "last_status_at": "2021-07-28",
+        "statuses_count": 9,
+        "last_status_at": "2024-11-01",
         "emojis": [],
         "fields": [
           {
@@ -245,7 +248,8 @@ func (suite *ReportsGetTestSuite) TestReportsGetAll() {
             "verified_at": null
           }
         ],
-        "hide_collections": true
+        "hide_collections": true,
+        "group": false
       },
       "created_by_application_id": "01F8MGY43H3N2C8EWPR2FPYEXG"
     },
@@ -286,6 +290,7 @@ func (suite *ReportsGetTestSuite) TestReportsGetAll() {
         "avatar_static": "",
         "header": "http://localhost:8080/assets/default_header.webp",
         "header_static": "http://localhost:8080/assets/default_header.webp",
+        "header_description": "Flat gray background (default header).",
         "followers_count": 1,
         "following_count": 1,
         "statuses_count": 4,
@@ -299,7 +304,8 @@ func (suite *ReportsGetTestSuite) TestReportsGetAll() {
             "name": "admin",
             "color": ""
           }
-        ]
+        ],
+        "group": false
       },
       "created_by_application_id": "01F8MGXQRHYF5QPMTMXP78QC2F"
     },
@@ -340,6 +346,7 @@ func (suite *ReportsGetTestSuite) TestReportsGetAll() {
         "avatar_static": "",
         "header": "http://localhost:8080/assets/default_header.webp",
         "header_static": "http://localhost:8080/assets/default_header.webp",
+        "header_description": "Flat gray background (default header).",
         "followers_count": 1,
         "following_count": 1,
         "statuses_count": 4,
@@ -353,7 +360,8 @@ func (suite *ReportsGetTestSuite) TestReportsGetAll() {
             "name": "admin",
             "color": ""
           }
-        ]
+        ],
+        "group": false
       },
       "created_by_application_id": "01F8MGXQRHYF5QPMTMXP78QC2F"
     },
@@ -407,10 +415,11 @@ func (suite *ReportsGetTestSuite) TestReportsGetAll() {
         "avatar_static": "",
         "header": "http://localhost:8080/assets/default_header.webp",
         "header_static": "http://localhost:8080/assets/default_header.webp",
+        "header_description": "Flat gray background (default header).",
         "followers_count": 1,
         "following_count": 1,
-        "statuses_count": 8,
-        "last_status_at": "2021-07-28",
+        "statuses_count": 9,
+        "last_status_at": "2024-11-01",
         "emojis": [],
         "fields": [
           {
@@ -424,7 +433,8 @@ func (suite *ReportsGetTestSuite) TestReportsGetAll() {
             "verified_at": null
           }
         ],
-        "hide_collections": true
+        "hide_collections": true,
+        "group": false
       },
       "created_by_application_id": "01F8MGY43H3N2C8EWPR2FPYEXG"
     },
@@ -465,12 +475,14 @@ func (suite *ReportsGetTestSuite) TestReportsGetAll() {
         "avatar_static": "",
         "header": "http://localhost:8080/assets/default_header.webp",
         "header_static": "http://localhost:8080/assets/default_header.webp",
+        "header_description": "Flat gray background (default header).",
         "followers_count": 0,
         "following_count": 0,
-        "statuses_count": 3,
-        "last_status_at": "2021-09-11",
+        "statuses_count": 4,
+        "last_status_at": "2024-11-01",
         "emojis": [],
-        "fields": []
+        "fields": [],
+        "group": false
       }
     },
     "assigned_account": null,
@@ -479,6 +491,7 @@ func (suite *ReportsGetTestSuite) TestReportsGetAll() {
       {
         "id": "01FVW7JHQFSFK166WWKR8CBA6M",
         "created_at": "2021-09-20T10:40:37.000Z",
+        "edited_at": null,
         "in_reply_to_id": null,
         "in_reply_to_account_id": null,
         "sensitive": false,
@@ -495,7 +508,7 @@ func (suite *ReportsGetTestSuite) TestReportsGetAll() {
         "muted": false,
         "bookmarked": false,
         "pinned": false,
-        "content": "dark souls status bot: \"thoughts of dog\"",
+        "content": "\u003cp\u003edark souls status bot: \"thoughts of dog\"\u003c/p\u003e",
         "reblog": null,
         "account": {
           "id": "01F8MH5ZK5VRH73AKHQM6Y9VNX",
@@ -512,12 +525,14 @@ func (suite *ReportsGetTestSuite) TestReportsGetAll() {
           "avatar_static": "",
           "header": "http://localhost:8080/assets/default_header.webp",
           "header_static": "http://localhost:8080/assets/default_header.webp",
+          "header_description": "Flat gray background (default header).",
           "followers_count": 0,
           "following_count": 0,
-          "statuses_count": 3,
-          "last_status_at": "2021-09-11",
+          "statuses_count": 4,
+          "last_status_at": "2024-11-01",
           "emojis": [],
-          "fields": []
+          "fields": [],
+          "group": false
         },
         "media_attachments": [
           {
@@ -657,10 +672,11 @@ func (suite *ReportsGetTestSuite) TestReportsGetCreatedByAccount() {
         "avatar_static": "",
         "header": "http://localhost:8080/assets/default_header.webp",
         "header_static": "http://localhost:8080/assets/default_header.webp",
+        "header_description": "Flat gray background (default header).",
         "followers_count": 1,
         "following_count": 1,
-        "statuses_count": 8,
-        "last_status_at": "2021-07-28",
+        "statuses_count": 9,
+        "last_status_at": "2024-11-01",
         "emojis": [],
         "fields": [
           {
@@ -674,7 +690,8 @@ func (suite *ReportsGetTestSuite) TestReportsGetCreatedByAccount() {
             "verified_at": null
           }
         ],
-        "hide_collections": true
+        "hide_collections": true,
+        "group": false
       },
       "created_by_application_id": "01F8MGY43H3N2C8EWPR2FPYEXG"
     },
@@ -715,12 +732,14 @@ func (suite *ReportsGetTestSuite) TestReportsGetCreatedByAccount() {
         "avatar_static": "",
         "header": "http://localhost:8080/assets/default_header.webp",
         "header_static": "http://localhost:8080/assets/default_header.webp",
+        "header_description": "Flat gray background (default header).",
         "followers_count": 0,
         "following_count": 0,
-        "statuses_count": 3,
-        "last_status_at": "2021-09-11",
+        "statuses_count": 4,
+        "last_status_at": "2024-11-01",
         "emojis": [],
-        "fields": []
+        "fields": [],
+        "group": false
       }
     },
     "assigned_account": null,
@@ -729,6 +748,7 @@ func (suite *ReportsGetTestSuite) TestReportsGetCreatedByAccount() {
       {
         "id": "01FVW7JHQFSFK166WWKR8CBA6M",
         "created_at": "2021-09-20T10:40:37.000Z",
+        "edited_at": null,
         "in_reply_to_id": null,
         "in_reply_to_account_id": null,
         "sensitive": false,
@@ -745,7 +765,7 @@ func (suite *ReportsGetTestSuite) TestReportsGetCreatedByAccount() {
         "muted": false,
         "bookmarked": false,
         "pinned": false,
-        "content": "dark souls status bot: \"thoughts of dog\"",
+        "content": "\u003cp\u003edark souls status bot: \"thoughts of dog\"\u003c/p\u003e",
         "reblog": null,
         "account": {
           "id": "01F8MH5ZK5VRH73AKHQM6Y9VNX",
@@ -762,12 +782,14 @@ func (suite *ReportsGetTestSuite) TestReportsGetCreatedByAccount() {
           "avatar_static": "",
           "header": "http://localhost:8080/assets/default_header.webp",
           "header_static": "http://localhost:8080/assets/default_header.webp",
+          "header_description": "Flat gray background (default header).",
           "followers_count": 0,
           "following_count": 0,
-          "statuses_count": 3,
-          "last_status_at": "2021-09-11",
+          "statuses_count": 4,
+          "last_status_at": "2024-11-01",
           "emojis": [],
-          "fields": []
+          "fields": [],
+          "group": false
         },
         "media_attachments": [
           {
@@ -907,10 +929,11 @@ func (suite *ReportsGetTestSuite) TestReportsGetTargetAccount() {
         "avatar_static": "",
         "header": "http://localhost:8080/assets/default_header.webp",
         "header_static": "http://localhost:8080/assets/default_header.webp",
+        "header_description": "Flat gray background (default header).",
         "followers_count": 1,
         "following_count": 1,
-        "statuses_count": 8,
-        "last_status_at": "2021-07-28",
+        "statuses_count": 9,
+        "last_status_at": "2024-11-01",
         "emojis": [],
         "fields": [
           {
@@ -924,7 +947,8 @@ func (suite *ReportsGetTestSuite) TestReportsGetTargetAccount() {
             "verified_at": null
           }
         ],
-        "hide_collections": true
+        "hide_collections": true,
+        "group": false
       },
       "created_by_application_id": "01F8MGY43H3N2C8EWPR2FPYEXG"
     },
@@ -965,12 +989,14 @@ func (suite *ReportsGetTestSuite) TestReportsGetTargetAccount() {
         "avatar_static": "",
         "header": "http://localhost:8080/assets/default_header.webp",
         "header_static": "http://localhost:8080/assets/default_header.webp",
+        "header_description": "Flat gray background (default header).",
         "followers_count": 0,
         "following_count": 0,
-        "statuses_count": 3,
-        "last_status_at": "2021-09-11",
+        "statuses_count": 4,
+        "last_status_at": "2024-11-01",
         "emojis": [],
-        "fields": []
+        "fields": [],
+        "group": false
       }
     },
     "assigned_account": null,
@@ -979,6 +1005,7 @@ func (suite *ReportsGetTestSuite) TestReportsGetTargetAccount() {
       {
         "id": "01FVW7JHQFSFK166WWKR8CBA6M",
         "created_at": "2021-09-20T10:40:37.000Z",
+        "edited_at": null,
         "in_reply_to_id": null,
         "in_reply_to_account_id": null,
         "sensitive": false,
@@ -995,7 +1022,7 @@ func (suite *ReportsGetTestSuite) TestReportsGetTargetAccount() {
         "muted": false,
         "bookmarked": false,
         "pinned": false,
-        "content": "dark souls status bot: \"thoughts of dog\"",
+        "content": "\u003cp\u003edark souls status bot: \"thoughts of dog\"\u003c/p\u003e",
         "reblog": null,
         "account": {
           "id": "01F8MH5ZK5VRH73AKHQM6Y9VNX",
@@ -1012,12 +1039,14 @@ func (suite *ReportsGetTestSuite) TestReportsGetTargetAccount() {
           "avatar_static": "",
           "header": "http://localhost:8080/assets/default_header.webp",
           "header_static": "http://localhost:8080/assets/default_header.webp",
+          "header_description": "Flat gray background (default header).",
           "followers_count": 0,
           "following_count": 0,
-          "statuses_count": 3,
-          "last_status_at": "2021-09-11",
+          "statuses_count": 4,
+          "last_status_at": "2024-11-01",
           "emojis": [],
-          "fields": []
+          "fields": [],
+          "group": false
         },
         "media_attachments": [
           {
@@ -1120,7 +1149,7 @@ func (suite *ReportsGetTestSuite) TestReportsGetNotAdmin() {
 	testToken := suite.testTokens["local_account_1"]
 	testUser := suite.testUsers["local_account_1"]
 
-	reports, _, err := suite.getReports(testAccount, testToken, testUser, http.StatusForbidden, `{"error":"Forbidden: user 01F8MGVGPHQ2D3P3X0454H54Z5 not an admin"}`, nil, "", "", "", "", "", 20)
+	reports, _, err := suite.getReports(testAccount, testToken, testUser, http.StatusForbidden, `{"error":"Forbidden: token has insufficient scope permission"}`, nil, "", "", "", "", "", 20)
 	suite.NoError(err)
 	suite.Empty(reports)
 }
