@@ -66,6 +66,14 @@ you'll make society more equitable for all if you're not careful! :hammer_sickle
 	// be a serialized string of almost any type, so we pick a
 	// nice serialized key size on the upper end of normal.
 	sizeofResultKey = 2 * sizeofIDStr
+
+	// exampleWebPushAuth is a Base64-encoded 16-byte random auth secret.
+	// This secret is consumed as Base64 by webpush-go.
+	exampleWebPushAuth = "ZVxqlt5fzVgmSz2aqiA2XQ=="
+
+	// exampleWebPushP256dh is a Base64-encoded DH P-256 public key.
+	// This secret is consumed as Base64 by webpush-go.
+	exampleWebPushP256dh = "OrpejO16gV97uBXew/T0I7YoUv/CX8fz0z4g8RrQ+edXJqQPjX3XVSo2P0HhcCpCOR1+Dzj5LFcK9jYNqX7SBg=="
 )
 
 var (
@@ -232,13 +240,12 @@ func sizeofAccount() uintptr {
 		DisplayName:             exampleUsername,
 		Note:                    exampleText,
 		NoteRaw:                 exampleText,
-		Memorial:                func() *bool { ok := false; return &ok }(),
+		MemorializedAt:          exampleTime,
 		CreatedAt:               exampleTime,
 		UpdatedAt:               exampleTime,
 		FetchedAt:               exampleTime,
-		Bot:                     func() *bool { ok := true; return &ok }(),
-		Locked:                  func() *bool { ok := true; return &ok }(),
-		Discoverable:            func() *bool { ok := false; return &ok }(),
+		Locked:                  util.Ptr(true),
+		Discoverable:            util.Ptr(false),
 		URI:                     exampleURI,
 		URL:                     exampleURI,
 		InboxURI:                exampleURI,
@@ -246,7 +253,7 @@ func sizeofAccount() uintptr {
 		FollowersURI:            exampleURI,
 		FollowingURI:            exampleURI,
 		FeaturedCollectionURI:   exampleURI,
-		ActorType:               ap.ActorPerson,
+		ActorType:               gtsmodel.AccountActorTypePerson,
 		PrivateKey:              &rsa.PrivateKey{},
 		PublicKey:               &rsa.PublicKey{},
 		PublicKeyURI:            exampleURI,
@@ -294,15 +301,14 @@ func sizeofAccountStats() uintptr {
 
 func sizeofApplication() uintptr {
 	return uintptr(size.Of(&gtsmodel.Application{
-		ID:           exampleID,
-		CreatedAt:    exampleTime,
-		UpdatedAt:    exampleTime,
-		Name:         exampleUsername,
-		Website:      exampleURI,
-		RedirectURI:  exampleURI,
-		ClientID:     exampleID,
-		ClientSecret: exampleID,
-		Scopes:       exampleTextSmall,
+		ID:              exampleID,
+		Name:            exampleUsername,
+		Website:         exampleURI,
+		RedirectURIs:    []string{exampleURI},
+		ClientID:        exampleID,
+		ClientSecret:    exampleID,
+		Scopes:          exampleTextSmall,
+		ManagedByUserID: exampleID,
 	}))
 }
 
@@ -317,17 +323,6 @@ func sizeofBlock() uintptr {
 	}))
 }
 
-func sizeofClient() uintptr {
-	return uintptr(size.Of(&gtsmodel.Client{
-		ID:        exampleID,
-		CreatedAt: exampleTime,
-		UpdatedAt: exampleTime,
-		Secret:    exampleID,
-		Domain:    exampleURI,
-		UserID:    exampleID,
-	}))
-}
-
 func sizeofConversation() uintptr {
 	return uintptr(size.Of(&gtsmodel.Conversation{
 		ID:               exampleID,
@@ -342,6 +337,41 @@ func sizeofConversation() uintptr {
 	}))
 }
 
+func sizeofDomainPermissionDraft() uintptr {
+	return uintptr(size.Of(&gtsmodel.DomainPermissionDraft{
+		ID:                 exampleID,
+		CreatedAt:          exampleTime,
+		UpdatedAt:          exampleTime,
+		PermissionType:     gtsmodel.DomainPermissionBlock,
+		Domain:             "example.org",
+		CreatedByAccountID: exampleID,
+		PrivateComment:     exampleTextSmall,
+		PublicComment:      exampleTextSmall,
+		Obfuscate:          util.Ptr(false),
+		SubscriptionID:     exampleID,
+	}))
+}
+
+func sizeofDomainPermissionSubscription() uintptr {
+	return uintptr(size.Of(&gtsmodel.DomainPermissionSubscription{
+		ID:                    exampleID,
+		Priority:              uint8(255),
+		Title:                 exampleTextSmall,
+		PermissionType:        gtsmodel.DomainPermissionBlock,
+		AsDraft:               util.Ptr(true),
+		CreatedByAccountID:    exampleID,
+		URI:                   exampleURI,
+		ContentType:           gtsmodel.DomainPermSubContentTypeCSV,
+		FetchUsername:         "username",
+		FetchPassword:         "password",
+		FetchedAt:             exampleTime,
+		SuccessfullyFetchedAt: exampleTime,
+		ETag:                  exampleID,
+		LastModified:          exampleTime,
+		Error:                 exampleTextSmall,
+	}))
+}
+
 func sizeofEmoji() uintptr {
 	return uintptr(size.Of(&gtsmodel.Emoji{
 		ID:                     exampleID,
@@ -490,7 +520,6 @@ func sizeofMedia() uintptr {
 		URL:               exampleURI,
 		RemoteURL:         exampleURI,
 		CreatedAt:         exampleTime,
-		UpdatedAt:         exampleTime,
 		Type:              gtsmodel.FileTypeImage,
 		AccountID:         exampleID,
 		Description:       exampleText,
@@ -517,7 +546,6 @@ func sizeofMention() uintptr {
 		ID:               exampleURI,
 		StatusID:         exampleURI,
 		CreatedAt:        exampleTime,
-		UpdatedAt:        exampleTime,
 		OriginAccountID:  exampleURI,
 		OriginAccountURI: exampleURI,
 		TargetAccountID:  exampleID,
@@ -543,7 +571,7 @@ func sizeofMove() uintptr {
 func sizeofNotification() uintptr {
 	return uintptr(size.Of(&gtsmodel.Notification{
 		ID:               exampleID,
-		NotificationType: gtsmodel.NotificationFave,
+		NotificationType: gtsmodel.NotificationFavourite,
 		CreatedAt:        exampleTime,
 		TargetAccountID:  exampleID,
 		OriginAccountID:  exampleID,
@@ -620,12 +648,13 @@ func sizeofStatus() uintptr {
 		URL:                      exampleURI,
 		Content:                  exampleText,
 		Text:                     exampleText,
+		ContentType:              gtsmodel.StatusContentTypePlain,
 		AttachmentIDs:            []string{exampleID, exampleID, exampleID},
 		TagIDs:                   []string{exampleID, exampleID, exampleID},
 		MentionIDs:               []string{},
 		EmojiIDs:                 []string{exampleID, exampleID, exampleID},
 		CreatedAt:                exampleTime,
-		UpdatedAt:                exampleTime,
+		EditedAt:                 exampleTime,
 		FetchedAt:                exampleTime,
 		Local:                    func() *bool { ok := false; return &ok }(),
 		AccountURI:               exampleURI,
@@ -636,6 +665,7 @@ func sizeofStatus() uintptr {
 		BoostOfID:                exampleID,
 		BoostOfAccountID:         exampleID,
 		ContentWarning:           exampleUsername, // similar length
+		ContentWarningText:       exampleUsername, // similar length
 		Visibility:               gtsmodel.VisibilityPublic,
 		Sensitive:                func() *bool { ok := false; return &ok }(),
 		Language:                 "en",
@@ -659,6 +689,24 @@ func sizeofStatusBookmark() uintptr {
 	}))
 }
 
+func sizeofStatusEdit() uintptr {
+	return uintptr(size.Of(&gtsmodel.StatusEdit{
+		ID:             exampleID,
+		Content:        exampleText,
+		ContentWarning: exampleUsername, // similar length
+		Text:           exampleText,
+		ContentType:    gtsmodel.StatusContentTypePlain,
+		Language:       "en",
+		Sensitive:      func() *bool { ok := false; return &ok }(),
+		AttachmentIDs:  []string{exampleID, exampleID, exampleID},
+		Attachments:    nil,
+		PollOptions:    []string{exampleTextSmall, exampleTextSmall, exampleTextSmall, exampleTextSmall},
+		PollVotes:      []int{69, 420, 1337, 1969},
+		StatusID:       exampleID,
+		CreatedAt:      exampleTime,
+	}))
+}
+
 func sizeofStatusFave() uintptr {
 	return uintptr(size.Of(&gtsmodel.StatusFave{
 		ID:              exampleID,
@@ -694,8 +742,7 @@ func sizeofThreadMute() uintptr {
 func sizeofToken() uintptr {
 	return uintptr(size.Of(&gtsmodel.Token{
 		ID:                  exampleID,
-		CreatedAt:           exampleTime,
-		UpdatedAt:           exampleTime,
+		LastUsed:            exampleTime,
 		ClientID:            exampleID,
 		UserID:              exampleID,
 		RedirectURI:         exampleURI,
@@ -771,3 +818,11 @@ func sizeofUserMute() uintptr {
 		Notifications:   util.Ptr(false),
 	}))
 }
+
+func sizeofWebPushSubscription() uintptr {
+	return uintptr(size.Of(&gtsmodel.WebPushSubscription{
+		TokenID: exampleID,
+		Auth:    exampleWebPushAuth,
+		P256dh:  exampleWebPushP256dh,
+	}))
+}
