@@ -51,11 +51,13 @@ type DB interface {
 	SinBinStatus
 	Status
 	StatusBookmark
+	StatusEdit
 	StatusFave
 	Tag
 	Thread
 	Timeline
 	User
 	Tombstone
+	WebPush
 	WorkerTask
 }
