@@ -35,6 +35,7 @@ import (
 	"github.com/superseriousbusiness/gotosocial/internal/log"
 	"github.com/superseriousbusiness/gotosocial/internal/media"
 	"github.com/superseriousbusiness/gotosocial/internal/util"
+	"github.com/superseriousbusiness/gotosocial/internal/util/xslices"
 )
 
 // statusFresh returns true if the given status is still
@@ -302,6 +303,7 @@ func (d *Dereferencer) enrichStatusSafely(
 		uri,
 		status,
 		statusable,
+		isNew,
 	)
 
 	// Check for a returned HTTP code via error.
@@ -374,6 +376,7 @@ func (d *Dereferencer) enrichStatus(
 	uri *url.URL,
 	status *gtsmodel.Status,
 	statusable ap.Statusable,
+	isNew bool,
 ) (
 	*gtsmodel.Status,
 	ap.Statusable,
@@ -451,7 +454,8 @@ func (d *Dereferencer) enrichStatus(
 
 	// Ensure we have the author account of the status dereferenced (+ up-to-date). If this is a new status
 	// (i.e. status.AccountID == "") then any error here is irrecoverable. status.AccountID must ALWAYS be set.
-	if _, _, err := d.getAccountByURI(ctx, requestUser, attributedTo); err != nil && status.AccountID == "" {
+	// We want the exact URI match here as well, not the imprecise URL match.
+	if _, _, err := d.getAccountByURI(ctx, requestUser, attributedTo, false); err != nil && status.AccountID == "" {
 
 		// Note that we specifically DO NOT wrap the error, instead collapsing it as string.
 		// Errors fetching an account do not necessarily relate to dereferencing the status.
@@ -476,8 +480,7 @@ func (d *Dereferencer) enrichStatus(
 
 	// Ensure the final parsed status URI or URL matches
 	// the input URI we fetched (or received) it as.
-	matches, err := util.URIMatches(
-		uri,
+	matches, err := util.URIMatches(uri,
 		append(
 			ap.GetURL(statusable),      // status URL(s)
 			ap.GetJSONLDId(statusable), // status URI
@@ -497,21 +500,18 @@ func (d *Dereferencer) enrichStatus(
 		)
 	}
 
-	var isNew bool
-
-	// Based on the original provided
-	// status model, determine whether
-	// this is a new insert / update.
-	if isNew = (status.ID == ""); isNew {
+	if isNew {
 
 		// Generate new status ID from the provided creation date.
-		latestStatus.ID, err = id.NewULIDFromTime(latestStatus.CreatedAt)
-		if err != nil {
-			log.Errorf(ctx, "invalid created at date (falling back to 'now'): %v", err)
-			latestStatus.ID = id.NewULID() // just use "now"
-		}
+		latestStatus.ID = id.NewULIDFromTime(latestStatus.CreatedAt)
 	} else {
 
+		// Ensure that status isn't trying to re-date itself.
+		if !latestStatus.CreatedAt.Equal(status.CreatedAt) {
+			err := gtserror.Newf("status %s 'published' changed", uri)
+			return nil, nil, gtserror.SetMalformed(err)
+		}
+
 		// Reuse existing status ID.
 		latestStatus.ID = status.ID
 	}
@@ -519,7 +519,6 @@ func (d *Dereferencer) enrichStatus(
 	// Set latest fetch time and carry-
 	// over some values from "old" status.
 	latestStatus.FetchedAt = time.Now()
-	latestStatus.UpdatedAt = status.UpdatedAt
 	latestStatus.Local = status.Local
 	latestStatus.PinnedAt = status.PinnedAt
 
@@ -538,8 +537,9 @@ func (d *Dereferencer) enrichStatus(
 	}
 
 	// Check if this is a permitted status we should accept.
-	// Function also sets "PendingApproval" bool as necessary.
-	permit, err := d.isPermittedStatus(ctx, requestUser, status, latestStatus)
+	// Function also sets "PendingApproval" bool as necessary,
+	// and handles removal of existing statuses no longer permitted.
+	permit, err := d.isPermittedStatus(ctx, requestUser, status, latestStatus, isNew)
 	if err != nil {
 		return nil, nil, gtserror.Newf("error checking permissibility for status %s: %w", uri, err)
 	}
@@ -550,59 +550,117 @@ func (d *Dereferencer) enrichStatus(
 		return nil, nil, gtserror.SetNotPermitted(err)
 	}
 
-	// Ensure the status' mentions are populated, and pass in existing to check for changes.
-	if err := d.fetchStatusMentions(ctx, requestUser, status, latestStatus); err != nil {
+	// Insert / update any attached status poll.
+	pollChanged, err := d.handleStatusPoll(ctx,
+		status,
+		latestStatus,
+	)
+	if err != nil {
+		return nil, nil, gtserror.Newf("error handling poll for status %s: %w", uri, err)
+	}
+
+	// Populate mentions associated with status, passing
+	// in existing status to reuse old where possible.
+	// (especially important here to reduce need to dereference).
+	mentionsChanged, err := d.fetchStatusMentions(ctx,
+		requestUser,
+		status,
+		latestStatus,
+	)
+	if err != nil {
 		return nil, nil, gtserror.Newf("error populating mentions for status %s: %w", uri, err)
 	}
 
-	// Ensure the status' poll remains consistent, else reset the poll.
-	if err := d.fetchStatusPoll(ctx, status, latestStatus); err != nil {
-		return nil, nil, gtserror.Newf("error populating poll for status %s: %w", uri, err)
+	// Ensure status in a thread is connected.
+	threadChanged, err := d.threadStatus(ctx,
+		status,
+		latestStatus,
+	)
+	if err != nil {
+		return nil, nil, gtserror.Newf("error handling threading for status %s: %w", uri, err)
 	}
 
-	// Now that we know who this status replies to (handled by ASStatusToStatus)
-	// and who it mentions, we can add a ThreadID to it if necessary.
-	if err := d.threadStatus(ctx, latestStatus); err != nil {
-		return nil, nil, gtserror.Newf("error checking / creating threadID for status %s: %w", uri, err)
-	}
-
-	// Ensure the status' tags are populated, (changes are expected / okay).
-	if err := d.fetchStatusTags(ctx, status, latestStatus); err != nil {
+	// Populate tags associated with status, passing
+	// in existing status to reuse old where possible.
+	tagsChanged, err := d.fetchStatusTags(ctx,
+		status,
+		latestStatus,
+	)
+	if err != nil {
 		return nil, nil, gtserror.Newf("error populating tags for status %s: %w", uri, err)
 	}
 
-	// Ensure the status' media attachments are populated, passing in existing to check for changes.
-	if err := d.fetchStatusAttachments(ctx, requestUser, status, latestStatus); err != nil {
+	// Populate media attachments associated with status,
+	// passing in existing status to reuse old where possible
+	// (especially important here to reduce need to dereference).
+	mediaChanged, err := d.fetchStatusAttachments(ctx,
+		requestUser,
+		status,
+		latestStatus,
+	)
+	if err != nil {
 		return nil, nil, gtserror.Newf("error populating attachments for status %s: %w", uri, err)
 	}
 
-	// Ensure the status' emoji attachments are populated, passing in existing to check for changes.
-	if err := d.fetchStatusEmojis(ctx, status, latestStatus); err != nil {
+	// Populate emoji associated with status, passing
+	// in existing status to reuse old where possible
+	// (especially important here to reduce need to dereference).
+	emojiChanged, err := d.fetchStatusEmojis(ctx,
+		status,
+		latestStatus,
+	)
+	if err != nil {
 		return nil, nil, gtserror.Newf("error populating emojis for status %s: %w", uri, err)
 	}
 
 	if isNew {
-		// This is new, put the status in the database.
-		err := d.state.DB.PutStatus(ctx, latestStatus)
-		if err != nil {
-			return nil, nil, gtserror.Newf("error putting in database: %w", err)
+		// Simplest case, insert this new status into the database.
+		if err := d.state.DB.PutStatus(ctx, latestStatus); err != nil {
+			return nil, nil, gtserror.Newf("error inserting new status %s: %w", uri, err)
 		}
 	} else {
-		// This is an existing status, update the model in the database.
-		if err := d.state.DB.UpdateStatus(ctx, latestStatus); err != nil {
-			return nil, nil, gtserror.Newf("error updating database: %w", err)
+		// Check for and handle any edits to status, inserting
+		// historical edit if necessary. Also determines status
+		// columns that need updating in below query.
+		cols, err := d.handleStatusEdit(ctx,
+			status,
+			latestStatus,
+			pollChanged,
+			mentionsChanged,
+			threadChanged,
+			tagsChanged,
+			mediaChanged,
+			emojiChanged,
+		)
+		if err != nil {
+			return nil, nil, gtserror.Newf("error handling edit for status %s: %w", uri, err)
+		}
+
+		// With returned changed columns, now update the existing status entry.
+		if err := d.state.DB.UpdateStatus(ctx, latestStatus, cols...); err != nil {
+			return nil, nil, gtserror.Newf("error updating existing status %s: %w", uri, err)
 		}
 	}
 
 	return latestStatus, statusable, nil
 }
 
+// fetchStatusMentions populates the mentions on 'status', creating
+// new where needed, or using unchanged mentions from 'existing' status.
 func (d *Dereferencer) fetchStatusMentions(
 	ctx context.Context,
 	requestUser string,
 	existing *gtsmodel.Status,
 	status *gtsmodel.Status,
-) error {
+) (
+	changed bool,
+	err error,
+) {
+
+	// Get most-recent modified time
+	// for use in new mention ULIDs.
+	updatedAt := status.UpdatedAt()
+
 	// Allocate new slice to take the yet-to-be created mention IDs.
 	status.MentionIDs = make([]string, len(status.Mentions))
 
@@ -610,12 +668,11 @@ func (d *Dereferencer) fetchStatusMentions(
 		var (
 			mention       = status.Mentions[i]
 			alreadyExists bool
-			err           error
 		)
 
 		// Search existing status for a mention already stored,
 		// else ensure new mention's target account is populated.
-		mention, alreadyExists, err = d.getPopulatedMention(ctx,
+		mention, alreadyExists, err = d.populateMentionTarget(ctx,
 			requestUser,
 			existing,
 			mention,
@@ -633,19 +690,16 @@ func (d *Dereferencer) fetchStatusMentions(
 			continue
 		}
 
-		// This mention didn't exist yet.
-		// Generate new ID according to status creation.
-		// TODO: update this to use "edited_at" when we add
-		//       support for edited status revision history.
-		mention.ID, err = id.NewULIDFromTime(status.CreatedAt)
-		if err != nil {
-			log.Errorf(ctx, "invalid created at date (falling back to 'now'): %v", err)
-			mention.ID = id.NewULID() // just use "now"
-		}
+		// Mark status as
+		// having changed.
+		changed = true
 
-		// Set known further mention details.
-		mention.CreatedAt = status.CreatedAt
-		mention.UpdatedAt = status.UpdatedAt
+		// This mention didn't exist yet.
+		// Generate new ID according to latest update.
+		mention.ID = id.NewULIDFromTime(updatedAt)
+
+		// Set further mention details.
+		mention.CreatedAt = updatedAt
 		mention.OriginAccount = status.Account
 		mention.OriginAccountID = status.AccountID
 		mention.OriginAccountURI = status.AccountURI
@@ -657,7 +711,7 @@ func (d *Dereferencer) fetchStatusMentions(
 
 		// Place the new mention into the database.
 		if err := d.state.DB.PutMention(ctx, mention); err != nil {
-			return gtserror.Newf("error putting mention in database: %w", err)
+			return changed, gtserror.Newf("error putting mention in database: %w", err)
 		}
 
 		// Set the *new* mention and ID.
@@ -678,17 +732,42 @@ func (d *Dereferencer) fetchStatusMentions(
 		i++
 	}
 
-	return nil
+	return changed, nil
 }
 
-func (d *Dereferencer) threadStatus(ctx context.Context, status *gtsmodel.Status) error {
-	if status.InReplyTo != nil {
-		if parentThreadID := status.InReplyTo.ThreadID; parentThreadID != "" {
-			// Simplest case: parent status
-			// is threaded, so inherit threadID.
-			status.ThreadID = parentThreadID
-			return nil
+// threadStatus ensures that given status is threaded correctly
+// where necessary. that is it will inherit a thread ID from the
+// existing copy if it is threaded correctly, else it will inherit
+// a thread ID from a parent with existing thread, else it will
+// generate a new thread ID if status mentions a local account.
+func (d *Dereferencer) threadStatus(
+	ctx context.Context,
+	existing *gtsmodel.Status,
+	status *gtsmodel.Status,
+) (
+	changed bool,
+	err error,
+) {
+
+	// Check for existing status
+	// that is already threaded.
+	if existing.ThreadID != "" {
+
+		// Existing is threaded correctly.
+		if existing.InReplyTo == nil ||
+			existing.InReplyTo.ThreadID == existing.ThreadID {
+			status.ThreadID = existing.ThreadID
+			return false, nil
 		}
+
+		// TODO: delete incorrect thread
+	}
+
+	// Check for existing parent to inherit threading from.
+	if inReplyTo := status.InReplyTo; inReplyTo != nil &&
+		inReplyTo.ThreadID != "" {
+		status.ThreadID = inReplyTo.ThreadID
+		return true, nil
 	}
 
 	// Parent wasn't threaded. If this
@@ -711,7 +790,7 @@ func (d *Dereferencer) threadStatus(ctx context.Context, status *gtsmodel.Status
 		// Status doesn't mention a
 		// local account, so we don't
 		// need to thread it.
-		return nil
+		return false, nil
 	}
 
 	// Status mentions a local account.
@@ -719,24 +798,30 @@ func (d *Dereferencer) threadStatus(ctx context.Context, status *gtsmodel.Status
 	// it to the status.
 	threadID := id.NewULID()
 
-	if err := d.state.DB.PutThread(
-		ctx,
-		&gtsmodel.Thread{
-			ID: threadID,
-		},
+	// Insert new thread model into db.
+	if err := d.state.DB.PutThread(ctx,
+		&gtsmodel.Thread{ID: threadID},
 	); err != nil {
-		return gtserror.Newf("error inserting new thread in db: %w", err)
+		return false, gtserror.Newf("error inserting new thread in db: %w", err)
 	}
 
+	// Set thread on latest status.
 	status.ThreadID = threadID
-	return nil
+	return true, nil
 }
 
+// fetchStatusTags populates the tags on 'status', fetching existing
+// from the database and creating new where needed. 'existing' is used
+// to fetch tags that have not changed since previous stored status.
 func (d *Dereferencer) fetchStatusTags(
 	ctx context.Context,
 	existing *gtsmodel.Status,
 	status *gtsmodel.Status,
-) error {
+) (
+	changed bool,
+	err error,
+) {
+
 	// Allocate new slice to take the yet-to-be determined tag IDs.
 	status.TagIDs = make([]string, len(status.Tags))
 
@@ -751,10 +836,14 @@ func (d *Dereferencer) fetchStatusTags(
 			continue
 		}
 
+		// Mark status as
+		// having changed.
+		changed = true
+
 		// Look for existing tag with name in the database.
 		existing, err := d.state.DB.GetTagByName(ctx, tag.Name)
 		if err != nil && !errors.Is(err, db.ErrNoEntries) {
-			return gtserror.Newf("db error getting tag %s: %w", tag.Name, err)
+			return changed, gtserror.Newf("db error getting tag %s: %w", tag.Name, err)
 		} else if existing != nil {
 			status.Tags[i] = existing
 			status.TagIDs[i] = existing.ID
@@ -788,106 +877,21 @@ func (d *Dereferencer) fetchStatusTags(
 		i++
 	}
 
-	return nil
-}
-
-func (d *Dereferencer) fetchStatusPoll(
-	ctx context.Context,
-	existing *gtsmodel.Status,
-	status *gtsmodel.Status,
-) error {
-	var (
-		// insertStatusPoll generates ID and inserts the poll attached to status into the database.
-		insertStatusPoll = func(ctx context.Context, status *gtsmodel.Status) error {
-			var err error
-
-			// Generate new ID for poll from the status CreatedAt.
-			// TODO: update this to use "edited_at" when we add
-			//       support for edited status revision history.
-			status.Poll.ID, err = id.NewULIDFromTime(status.CreatedAt)
-			if err != nil {
-				log.Errorf(ctx, "invalid created at date (falling back to 'now'): %v", err)
-				status.Poll.ID = id.NewULID() // just use "now"
-			}
-
-			// Update the status<->poll links.
-			status.PollID = status.Poll.ID
-			status.Poll.StatusID = status.ID
-			status.Poll.Status = status
-
-			// Insert this latest poll into the database.
-			err = d.state.DB.PutPoll(ctx, status.Poll)
-			if err != nil {
-				return gtserror.Newf("error putting in database: %w", err)
-			}
-
-			return nil
-		}
-
-		// deleteStatusPoll deletes the poll with ID, and all attached votes, from the database.
-		deleteStatusPoll = func(ctx context.Context, pollID string) error {
-			if err := d.state.DB.DeletePollByID(ctx, pollID); err != nil {
-				return gtserror.Newf("error deleting existing poll from database: %w", err)
-			}
-			return nil
-		}
-	)
-
-	switch {
-	case existing.Poll == nil && status.Poll == nil:
-		// no poll before or after, nothing to do.
-		return nil
-
-	case existing.Poll == nil && status.Poll != nil:
-		// no previous poll, insert new poll!
-		return insertStatusPoll(ctx, status)
-
-	case status.Poll == nil:
-		// existing poll has been deleted, remove this.
-		return deleteStatusPoll(ctx, existing.PollID)
-
-	case pollChanged(existing.Poll, status.Poll):
-		// poll has changed since original, delete and reinsert new.
-		if err := deleteStatusPoll(ctx, existing.PollID); err != nil {
-			return err
-		}
-		return insertStatusPoll(ctx, status)
-
-	case pollUpdated(existing.Poll, status.Poll):
-		// Since we last saw it, the poll has updated!
-		// Whether that be stats, or close time.
-		poll := existing.Poll
-		poll.Closing = pollJustClosed(existing.Poll, status.Poll)
-		poll.ClosedAt = status.Poll.ClosedAt
-		poll.Voters = status.Poll.Voters
-		poll.Votes = status.Poll.Votes
-
-		// Update poll model in the database (specifically only the possible changed columns).
-		if err := d.state.DB.UpdatePoll(ctx, poll, "closed_at", "voters", "votes"); err != nil {
-			return gtserror.Newf("error updating poll: %w", err)
-		}
-
-		// Update poll on status.
-		status.PollID = poll.ID
-		status.Poll = poll
-		return nil
-
-	default:
-		// latest and existing
-		// polls are up to date.
-		poll := existing.Poll
-		status.PollID = poll.ID
-		status.Poll = poll
-		return nil
-	}
+	return changed, nil
 }
 
+// fetchStatusAttachments populates the attachments on 'status', creating new database
+// entries where needed and dereferencing it, or using unchanged from 'existing' status.
 func (d *Dereferencer) fetchStatusAttachments(
 	ctx context.Context,
 	requestUser string,
 	existing *gtsmodel.Status,
 	status *gtsmodel.Status,
-) error {
+) (
+	changed bool,
+	err error,
+) {
+
 	// Allocate new slice to take the yet-to-be fetched attachment IDs.
 	status.AttachmentIDs = make([]string, len(status.Attachments))
 
@@ -897,9 +901,26 @@ func (d *Dereferencer) fetchStatusAttachments(
 		// Look for existing media attachment with remote URL first.
 		existing, ok := existing.GetAttachmentByRemoteURL(placeholder.RemoteURL)
 		if ok && existing.ID != "" {
+			var info media.AdditionalMediaInfo
 
-			// Ensure the existing media attachment is up-to-date and cached.
-			existing, err := d.updateAttachment(ctx, requestUser, existing, placeholder)
+			// Look for any difference in stored media description.
+			diff := (existing.Description != placeholder.Description)
+			if diff {
+				info.Description = &placeholder.Description
+			}
+
+			// If description changed,
+			// we mark media as changed.
+			changed = changed || diff
+
+			// Store any attachment updates and
+			// ensure media is locally cached.
+			existing, err := d.RefreshMedia(ctx,
+				requestUser,
+				existing,
+				info,
+				diff,
+			)
 			if err != nil {
 				log.Errorf(ctx, "error updating existing attachment: %v", err)
 
@@ -915,9 +936,12 @@ func (d *Dereferencer) fetchStatusAttachments(
 			continue
 		}
 
+		// Mark status as
+		// having changed.
+		changed = true
+
 		// Load this new media attachment.
-		attachment, err := d.GetMedia(
-			ctx,
+		attachment, err := d.GetMedia(ctx,
 			requestUser,
 			status.AccountID,
 			placeholder.RemoteURL,
@@ -955,43 +979,319 @@ func (d *Dereferencer) fetchStatusAttachments(
 		i++
 	}
 
-	return nil
+	return changed, nil
 }
 
+// fetchStatusEmojis populates the emojis on 'status', creating new database entries
+// where needed and dereferencing it, or using unchanged from 'existing' status.
 func (d *Dereferencer) fetchStatusEmojis(
 	ctx context.Context,
 	existing *gtsmodel.Status,
 	status *gtsmodel.Status,
-) error {
+) (
+	changed bool,
+	err error,
+) {
+
 	// Fetch the updated emojis for our status.
 	emojis, changed, err := d.fetchEmojis(ctx,
 		existing.Emojis,
 		status.Emojis,
 	)
 	if err != nil {
-		return gtserror.Newf("error fetching emojis: %w", err)
+		return changed, gtserror.Newf("error fetching emojis: %w", err)
 	}
 
 	if !changed {
 		// Use existing status emoji objects.
 		status.EmojiIDs = existing.EmojiIDs
 		status.Emojis = existing.Emojis
-		return nil
+		return false, nil
 	}
 
 	// Set latest emojis.
 	status.Emojis = emojis
 
-	// Iterate over and set changed emoji IDs.
+	// Extract IDs from latest slice of emojis.
 	status.EmojiIDs = make([]string, len(emojis))
 	for i, emoji := range emojis {
 		status.EmojiIDs[i] = emoji.ID
 	}
 
+	// Combine both old and new emojis, as statuses.emojis
+	// keeps track of emojis for both old and current edits.
+	status.EmojiIDs = append(status.EmojiIDs, existing.EmojiIDs...)
+	status.Emojis = append(status.Emojis, existing.Emojis...)
+	status.EmojiIDs = xslices.Deduplicate(status.EmojiIDs)
+	status.Emojis = xslices.DeduplicateFunc(status.Emojis,
+		func(e *gtsmodel.Emoji) string { return e.ID },
+	)
+
+	return true, nil
+}
+
+// handleStatusPoll handles both inserting of new status poll or the
+// update of an existing poll. this handles the case of simple vote
+// count updates (without being classified as a change of the poll
+// itself), as well as full poll changes that delete existing instance.
+func (d *Dereferencer) handleStatusPoll(
+	ctx context.Context,
+	existing *gtsmodel.Status,
+	status *gtsmodel.Status,
+) (
+	changed bool,
+	err error,
+) {
+	switch {
+	case existing.Poll == nil && status.Poll == nil:
+		// no poll before or after, nothing to do.
+		return false, nil
+
+	case existing.Poll == nil && status.Poll != nil:
+		// no previous poll, insert new status poll!
+		return true, d.insertStatusPoll(ctx, status)
+
+	case status.Poll == nil:
+		// existing status poll has been deleted, remove this from the database.
+		if err = d.state.DB.DeletePollByID(ctx, existing.Poll.ID); err != nil {
+			err = gtserror.Newf("error deleting poll from database: %w", err)
+		}
+		return true, err
+
+	case pollChanged(existing.Poll, status.Poll):
+		// existing status poll has been changed, remove this from the database.
+		if err = d.state.DB.DeletePollByID(ctx, existing.Poll.ID); err != nil {
+			return true, gtserror.Newf("error deleting poll from database: %w", err)
+		}
+
+		// insert latest poll version into database.
+		return true, d.insertStatusPoll(ctx, status)
+
+	case pollStateUpdated(existing.Poll, status.Poll):
+		// Since we last saw it, the poll has updated!
+		// Whether that be stats, or close time.
+		poll := existing.Poll
+		poll.Closing = pollJustClosed(existing.Poll, status.Poll)
+		poll.ClosedAt = status.Poll.ClosedAt
+		poll.Voters = status.Poll.Voters
+		poll.Votes = status.Poll.Votes
+
+		// Update poll model in the database (specifically only the possible changed columns).
+		if err = d.state.DB.UpdatePoll(ctx, poll, "closed_at", "voters", "votes"); err != nil {
+			return false, gtserror.Newf("error updating poll: %w", err)
+		}
+
+		// Update poll on status.
+		status.PollID = poll.ID
+		status.Poll = poll
+		return false, nil
+
+	default:
+		// latest and existing
+		// polls are up to date.
+		poll := existing.Poll
+		status.PollID = poll.ID
+		status.Poll = poll
+		return false, nil
+	}
+}
+
+// insertStatusPoll inserts an assumed new poll attached to status into the database, this
+// also handles generating new ID for the poll and setting necessary fields on the status.
+func (d *Dereferencer) insertStatusPoll(ctx context.Context, status *gtsmodel.Status) error {
+	var err error
+
+	// Get most-recent modified time
+	// which will be poll creation time.
+	createdAt := status.UpdatedAt()
+
+	// Generate new ID for poll from createdAt.
+	status.Poll.ID = id.NewULIDFromTime(createdAt)
+
+	// Update the status<->poll links.
+	status.PollID = status.Poll.ID
+	status.Poll.StatusID = status.ID
+	status.Poll.Status = status
+
+	// Insert this latest poll into the database.
+	err = d.state.DB.PutPoll(ctx, status.Poll)
+	if err != nil {
+		return gtserror.Newf("error putting poll in database: %w", err)
+	}
+
 	return nil
 }
 
-// getPopulatedMention tries to populate the given
+// handleStatusEdit compiles a list of changed status table columns between
+// existing and latest status model, and where necessary inserts a historic
+// edit of the status into the database to store its previous state. the
+// returned slice is a list of columns requiring updating in the database.
+func (d *Dereferencer) handleStatusEdit(
+	ctx context.Context,
+	existing *gtsmodel.Status,
+	status *gtsmodel.Status,
+	pollChanged bool,
+	mentionsChanged bool,
+	threadChanged bool,
+	tagsChanged bool,
+	mediaChanged bool,
+	emojiChanged bool,
+) (
+	cols []string,
+	err error,
+) {
+	var edited bool
+
+	// Copy previous status edit columns.
+	status.EditIDs = existing.EditIDs
+	status.Edits = existing.Edits
+
+	// Preallocate max slice length.
+	cols = make([]string, 1, 13)
+
+	// Always update `fetched_at`.
+	cols[0] = "fetched_at"
+
+	// Check for edited status content.
+	if existing.Content != status.Content {
+		cols = append(cols, "content")
+		edited = true
+	}
+
+	// Check for edited status content warning.
+	if existing.ContentWarning != status.ContentWarning {
+		cols = append(cols, "content_warning")
+		edited = true
+	}
+
+	// Check for edited status sensitive flag.
+	if *existing.Sensitive != *status.Sensitive {
+		cols = append(cols, "sensitive")
+		edited = true
+	}
+
+	// Check for edited status language tag.
+	if existing.Language != status.Language {
+		cols = append(cols, "language")
+		edited = true
+	}
+
+	if pollChanged {
+		// Attached poll was changed.
+		cols = append(cols, "poll_id")
+		edited = true
+	}
+
+	if mentionsChanged {
+		cols = append(cols, "mentions") // i.e. MentionIDs
+
+		// Mentions changed doesn't necessarily
+		// indicate an edit, it may just not have
+		// been previously populated properly.
+	}
+
+	if threadChanged {
+		cols = append(cols, "thread_id")
+
+		// Thread changed doesn't necessarily
+		// indicate an edit, it may just now
+		// actually be included in a thread.
+	}
+
+	if tagsChanged {
+		cols = append(cols, "tags") // i.e. TagIDs
+
+		// Tags changed doesn't necessarily
+		// indicate an edit, it may just not have
+		// been previously populated properly.
+	}
+
+	if mediaChanged {
+		// Attached media was changed.
+		cols = append(cols, "attachments") // i.e. AttachmentIDs
+		edited = true
+	}
+
+	if emojiChanged {
+		// Attached emojis changed.
+		cols = append(cols, "emojis") // i.e. EmojiIDs
+
+		// We specifically store both *new* AND *old* edit
+		// revision emojis in the statuses.emojis column.
+		emojiByID := func(e *gtsmodel.Emoji) string { return e.ID }
+		status.Emojis = append(status.Emojis, existing.Emojis...)
+		status.Emojis = xslices.DeduplicateFunc(status.Emojis, emojiByID)
+		status.EmojiIDs = xslices.Gather(status.EmojiIDs[:0], status.Emojis, emojiByID)
+
+		// Emojis changed doesn't necessarily
+		// indicate an edit, it may just not have
+		// been previously populated properly.
+	}
+
+	if edited {
+		// Get previous-most-recent modified time,
+		// which will be this edit's creation time.
+		createdAt := existing.UpdatedAt()
+
+		// Status has been editted since last
+		// we saw it, take snapshot of existing.
+		var edit gtsmodel.StatusEdit
+		edit.ID = id.NewULIDFromTime(createdAt)
+		edit.Content = existing.Content
+		edit.ContentWarning = existing.ContentWarning
+		edit.Text = existing.Text
+		edit.ContentType = existing.ContentType
+		edit.Language = existing.Language
+		edit.Sensitive = existing.Sensitive
+		edit.StatusID = status.ID
+		edit.CreatedAt = createdAt
+
+		// Copy existing attachments and descriptions.
+		edit.AttachmentIDs = existing.AttachmentIDs
+		edit.Attachments = existing.Attachments
+		if l := len(existing.Attachments); l > 0 {
+			edit.AttachmentDescriptions = make([]string, l)
+			for i, attach := range existing.Attachments {
+				edit.AttachmentDescriptions[i] = attach.Description
+			}
+		}
+
+		if existing.Poll != nil {
+			// Poll only set if existing contained them.
+			edit.PollOptions = existing.Poll.Options
+
+			if pollChanged || !*existing.Poll.HideCounts ||
+				!existing.Poll.ClosedAt.IsZero() {
+				// If the counts are allowed to be
+				// shown, or poll has changed, then
+				// include poll vote counts in edit.
+				edit.PollVotes = existing.Poll.Votes
+			}
+		}
+
+		// Insert this new edit of existing status into database.
+		if err := d.state.DB.PutStatusEdit(ctx, &edit); err != nil {
+			return nil, gtserror.Newf("error putting edit in database: %w", err)
+		}
+
+		// Add edit to list of edits on the status.
+		status.EditIDs = append(status.EditIDs, edit.ID)
+		status.Edits = append(status.Edits, &edit)
+
+		// Add edit to list of cols.
+		cols = append(cols, "edits")
+	}
+
+	if !existing.EditedAt.Equal(status.EditedAt) {
+		// Whether status edited or not,
+		// edited_at column has changed.
+		cols = append(cols, "edited_at")
+	}
+
+	return cols, nil
+}
+
+// populateMentionTarget tries to populate the given
 // mention with the correct TargetAccount and (if not
 // yet set) TargetAccountURI, returning the populated
 // mention.
@@ -1003,7 +1303,13 @@ func (d *Dereferencer) fetchStatusEmojis(
 // Otherwise, this function will try to parse first
 // the Href of the mention, and then the namestring,
 // to see who it targets, and go fetch that account.
-func (d *Dereferencer) getPopulatedMention(
+//
+// Note: Ordinarily it would make sense to try the
+// namestring first, as it definitely can't be a URL
+// rather than a URI, but because some remotes do
+// silly things like only provide `@username` instead
+// of `@username@domain`, we try by URI first.
+func (d *Dereferencer) populateMentionTarget(
 	ctx context.Context,
 	requestUser string,
 	existing *gtsmodel.Status,
@@ -1013,8 +1319,9 @@ func (d *Dereferencer) getPopulatedMention(
 	bool, // True if mention already exists in the DB.
 	error,
 ) {
-	// Mentions can be created using Name or Href.
-	// Prefer Href (TargetAccountURI), fall back to Name.
+	// Mentions can be created using `name` or `href`.
+	//
+	// Prefer `href` (TargetAccountURI), fall back to Name.
 	if mention.TargetAccountURI != "" {
 
 		// Look for existing mention with target account's URI, if so use this.
@@ -1024,19 +1331,24 @@ func (d *Dereferencer) getPopulatedMention(
 		}
 
 		// Ensure that mention account URI is parseable.
-		accountURI, err := url.Parse(mention.TargetAccountURI)
+		targetAccountURI, err := url.Parse(mention.TargetAccountURI)
 		if err != nil {
 			err := gtserror.Newf("invalid account uri %q: %w", mention.TargetAccountURI, err)
 			return nil, false, err
 		}
 
-		// Ensure we have  account of the mention target dereferenced.
+		// Ensure we have the account of
+		// the mention target dereferenced.
+		//
+		// Use exact URI match only, not URL,
+		// as we want to be precise here.
 		mention.TargetAccount, _, err = d.getAccountByURI(ctx,
 			requestUser,
-			accountURI,
+			targetAccountURI,
+			false,
 		)
 		if err != nil {
-			err := gtserror.Newf("failed to dereference account %s: %w", accountURI, err)
+			err := gtserror.Newf("failed to dereference account %s: %w", targetAccountURI, err)
 			return nil, false, err
 		}
 	} else {
@@ -1054,17 +1366,32 @@ func (d *Dereferencer) getPopulatedMention(
 			return existingMention, true, nil
 		}
 
-		// Ensure we have the account of the mention target dereferenced.
+		// Ensure we have the account of
+		// the mention target dereferenced.
+		//
+		// This might fail if the remote does
+		// something silly like only setting
+		// `@username` and not `@username@domain`.
 		mention.TargetAccount, _, err = d.getAccountByUsernameDomain(ctx,
 			requestUser,
 			username,
 			domain,
 		)
-		if err != nil {
+		if err != nil && !errors.Is(err, db.ErrNoEntries) {
 			err := gtserror.Newf("failed to dereference account %s: %w", mention.NameString, err)
 			return nil, false, err
 		}
 
+		if mention.TargetAccount == nil {
+			// Probably failed for abovementioned
+			// silly reason. Nothing we can do about it.
+			err := gtserror.Newf(
+				"failed to populate mention target account (badly formatted namestring?) %s: %w",
+				mention.NameString, err,
+			)
+			return nil, false, err
+		}
+
 		// Look for existing mention with target account's URI, if so use this.
 		existingMention, ok = existing.GetMentionByTargetURI(mention.TargetAccountURI)
 		if ok && existingMention.ID != "" {
