@@ -22,6 +22,7 @@ import (
 	"context"
 	"errors"
 	"net/url"
+	"time"
 
 	"github.com/miekg/dns"
 	"github.com/superseriousbusiness/gotosocial/internal/ap"
@@ -69,19 +70,10 @@ func (c *Converter) ASRepresentationToAccount(
 	acct.URI = uri
 
 	// Check whether account is a usable actor type.
-	switch acct.ActorType = accountable.GetTypeName(); acct.ActorType {
-
-	// people, groups, and organizations aren't bots
-	case ap.ActorPerson, ap.ActorGroup, ap.ActorOrganization:
-		acct.Bot = util.Ptr(false)
-
-	// apps and services are
-	case ap.ActorApplication, ap.ActorService:
-		acct.Bot = util.Ptr(true)
-
-	// we don't know what this is!
-	default:
-		err := gtserror.Newf("unusable actor type for %s", uri)
+	actorTypeName := accountable.GetTypeName()
+	acct.ActorType = gtsmodel.ParseAccountActorType(actorTypeName)
+	if acct.ActorType == gtsmodel.AccountActorTypeUnknown {
+		err := gtserror.Newf("unusable actor type %s for %s", actorTypeName, uri)
 		return nil, gtserror.SetMalformed(err)
 	}
 
@@ -111,6 +103,13 @@ func (c *Converter) ASRepresentationToAccount(
 		acct.UpdatedAt = pub
 	}
 
+	// Extract updated time if possible, i.e. last edited.
+	if upd := ap.GetUpdated(accountable); !upd.IsZero() {
+		acct.UpdatedAt = upd
+	} else {
+		acct.UpdatedAt = acct.CreatedAt
+	}
+
 	// Extract a preferred name (display name), fallback to username.
 	if displayName := ap.ExtractName(accountable); displayName != "" {
 		acct.DisplayName = displayName
@@ -141,7 +140,7 @@ func (c *Converter) ASRepresentationToAccount(
 	}
 
 	// account emojis (used in bio, display name, fields)
-	acct.Emojis, err = ap.ExtractEmojis(accountable)
+	acct.Emojis, err = ap.ExtractEmojis(accountable, acct.Domain)
 	if err != nil {
 		log.Warnf(ctx, "error(s) extracting account emojis for %s: %v", uri, err)
 	}
@@ -153,9 +152,9 @@ func (c *Converter) ASRepresentationToAccount(
 	acct.Note = ap.ExtractSummary(accountable)
 
 	// Assume not memorial (todo)
-	acct.Memorial = util.Ptr(false)
+	acct.MemorializedAt = time.Time{}
 
-	// Extract 'manuallyApprovesFollowers' aka locked account (default = true).
+	// Extract 'manuallyApprovesFollowers' aka locked account (default = false).
 	manuallyApprovesFollowers := ap.GetManuallyApprovesFollowers(accountable)
 	acct.Locked = &manuallyApprovesFollowers
 
@@ -317,7 +316,7 @@ func (c *Converter) ASStatusToStatus(ctx context.Context, statusable ap.Statusab
 	// status.Emojis
 	//
 	// Custom emojis for later dereferencing.
-	if emojis, err := ap.ExtractEmojis(statusable); err != nil {
+	if emojis, err := ap.ExtractEmojis(statusable, uriObj.Host); err != nil {
 		log.Warnf(ctx, "error extracting emojis for %s: %v", uri, err)
 	} else {
 		status.Emojis = emojis
@@ -348,18 +347,29 @@ func (c *Converter) ASStatusToStatus(ctx context.Context, statusable ap.Statusab
 	// zero-time will fall back to db defaults.
 	if pub := ap.GetPublished(statusable); !pub.IsZero() {
 		status.CreatedAt = pub
-		status.UpdatedAt = pub
 	} else {
 		log.Warnf(ctx, "unusable published property on %s", uri)
+		status.CreatedAt = time.Now()
+	}
+
+	// status.Edited
+	//
+	// Extract and validate update time for status. Defaults to none.
+	if upd := ap.GetUpdated(statusable); !upd.Before(status.CreatedAt) {
+		status.EditedAt = upd
+	} else if !upd.IsZero() {
+
+		// This is a malformed status that will likely break our systems.
+		err := gtserror.Newf("status %s 'updated' predates 'published'", uri)
+		return nil, gtserror.SetMalformed(err)
 	}
 
 	// status.AccountURI
 	// status.AccountID
 	// status.Account
 	//
-	// Account that created the status. Assume we have
-	// this in the db by the time this function is called,
-	// error if we don't.
+	// Account that created the status. Assume we have this
+	// in the db by the time this function is called, else error.
 	status.Account, err = c.getASAttributedToAccount(ctx,
 		status.URI,
 		statusable,
@@ -415,9 +425,10 @@ func (c *Converter) ASStatusToStatus(ctx context.Context, statusable ap.Statusab
 		return nil, gtserror.SetMalformed(err)
 	}
 
-	// Status was sent to us or dereffed
-	// by us so it must be federated.
+	// Status was sent to us or dereffed by
+	// us so it must be federated and not local.
 	status.Federated = util.Ptr(true)
+	status.Local = util.Ptr(false)
 
 	// Derive interaction policy for this status.
 	status.InteractionPolicy = ap.ExtractInteractionPolicy(
@@ -427,9 +438,11 @@ func (c *Converter) ASStatusToStatus(ctx context.Context, statusable ap.Statusab
 
 	// Set approvedByURI if present,
 	// for later dereferencing.
-	approvedByURI := ap.GetApprovedBy(statusable)
-	if approvedByURI != nil {
-		status.ApprovedByURI = approvedByURI.String()
+	if ipa, ok := statusable.(ap.InteractionPolicyAware); ok {
+		approvedByURI := ap.GetApprovedBy(ipa)
+		if approvedByURI != nil {
+			status.ApprovedByURI = approvedByURI.String()
+		}
 	}
 
 	// Assume not pending approval; this may
@@ -628,9 +641,9 @@ func (c *Converter) ASAnnounceToStatus(
 	// zero-time will fall back to db defaults.
 	if pub := ap.GetPublished(announceable); !pub.IsZero() {
 		boost.CreatedAt = pub
-		boost.UpdatedAt = pub
 	} else {
 		log.Warnf(ctx, "unusable published property on %s", uri)
+		boost.CreatedAt = time.Now()
 	}
 
 	// Extract and load the boost actor account,
