@@ -26,9 +26,9 @@ import (
 	"net/url"
 	"strings"
 
-	"github.com/superseriousbusiness/activity/pub"
-	"github.com/superseriousbusiness/activity/streams"
-	"github.com/superseriousbusiness/activity/streams/vocab"
+	"codeberg.org/superseriousbusiness/activity/pub"
+	"codeberg.org/superseriousbusiness/activity/streams"
+	"codeberg.org/superseriousbusiness/activity/streams/vocab"
 	"github.com/superseriousbusiness/gotosocial/internal/ap"
 	"github.com/superseriousbusiness/gotosocial/internal/config"
 	"github.com/superseriousbusiness/gotosocial/internal/db"
@@ -36,12 +36,35 @@ import (
 	"github.com/superseriousbusiness/gotosocial/internal/gtsmodel"
 	"github.com/superseriousbusiness/gotosocial/internal/log"
 	"github.com/superseriousbusiness/gotosocial/internal/uris"
-	"github.com/superseriousbusiness/gotosocial/internal/util"
+	"github.com/superseriousbusiness/gotosocial/internal/util/xslices"
 )
 
-// AccountToAS converts a gts model account into an activity streams person, suitable for federation
-func (c *Converter) AccountToAS(ctx context.Context, a *gtsmodel.Account) (vocab.ActivityStreamsPerson, error) {
-	person := streams.NewActivityStreamsPerson()
+func accountableForActorType(actorType gtsmodel.AccountActorType) ap.Accountable {
+	switch actorType {
+	case gtsmodel.AccountActorTypeApplication:
+		return streams.NewActivityStreamsApplication()
+	case gtsmodel.AccountActorTypeGroup:
+		return streams.NewActivityStreamsGroup()
+	case gtsmodel.AccountActorTypeOrganization:
+		return streams.NewActivityStreamsOrganization()
+	case gtsmodel.AccountActorTypePerson:
+		return streams.NewActivityStreamsPerson()
+	case gtsmodel.AccountActorTypeService:
+		return streams.NewActivityStreamsService()
+	default:
+		panic("invalid actor type")
+	}
+}
+
+// AccountToAS converts a gts model
+// account into an accountable.
+func (c *Converter) AccountToAS(
+	ctx context.Context,
+	a *gtsmodel.Account,
+) (ap.Accountable, error) {
+	// Use appropriate underlying
+	// actor type of accountable.
+	accountable := accountableForActorType(a.ActorType)
 
 	// id should be the activitypub URI of this user
 	// something like https://example.org/users/example_user
@@ -51,7 +74,13 @@ func (c *Converter) AccountToAS(ctx context.Context, a *gtsmodel.Account) (vocab
 	}
 	idProp := streams.NewJSONLDIdProperty()
 	idProp.SetIRI(profileIDURI)
-	person.SetJSONLDId(idProp)
+	accountable.SetJSONLDId(idProp)
+
+	// published
+	// The moment when the account was created.
+	publishedProp := streams.NewActivityStreamsPublishedProperty()
+	publishedProp.Set(a.CreatedAt)
+	accountable.SetActivityStreamsPublished(publishedProp)
 
 	// following
 	// The URI for retrieving a list of accounts this user is following
@@ -61,7 +90,7 @@ func (c *Converter) AccountToAS(ctx context.Context, a *gtsmodel.Account) (vocab
 	}
 	followingProp := streams.NewActivityStreamsFollowingProperty()
 	followingProp.SetIRI(followingURI)
-	person.SetActivityStreamsFollowing(followingProp)
+	accountable.SetActivityStreamsFollowing(followingProp)
 
 	// followers
 	// The URI for retrieving a list of this user's followers
@@ -71,7 +100,7 @@ func (c *Converter) AccountToAS(ctx context.Context, a *gtsmodel.Account) (vocab
 	}
 	followersProp := streams.NewActivityStreamsFollowersProperty()
 	followersProp.SetIRI(followersURI)
-	person.SetActivityStreamsFollowers(followersProp)
+	accountable.SetActivityStreamsFollowers(followersProp)
 
 	// inbox
 	// the activitypub inbox of this user for accepting messages
@@ -81,7 +110,7 @@ func (c *Converter) AccountToAS(ctx context.Context, a *gtsmodel.Account) (vocab
 	}
 	inboxProp := streams.NewActivityStreamsInboxProperty()
 	inboxProp.SetIRI(inboxURI)
-	person.SetActivityStreamsInbox(inboxProp)
+	accountable.SetActivityStreamsInbox(inboxProp)
 
 	// shared inbox -- only add this if we know for sure it has one
 	if a.SharedInboxURI != nil && *a.SharedInboxURI != "" {
@@ -95,7 +124,7 @@ func (c *Converter) AccountToAS(ctx context.Context, a *gtsmodel.Account) (vocab
 		sharedInboxProp.SetIRI(sharedInboxURI)
 		endpoints.SetActivityStreamsSharedInbox(sharedInboxProp)
 		endpointsProp.AppendActivityStreamsEndpoints(endpoints)
-		person.SetActivityStreamsEndpoints(endpointsProp)
+		accountable.SetActivityStreamsEndpoints(endpointsProp)
 	}
 
 	// outbox
@@ -106,7 +135,7 @@ func (c *Converter) AccountToAS(ctx context.Context, a *gtsmodel.Account) (vocab
 	}
 	outboxProp := streams.NewActivityStreamsOutboxProperty()
 	outboxProp.SetIRI(outboxURI)
-	person.SetActivityStreamsOutbox(outboxProp)
+	accountable.SetActivityStreamsOutbox(outboxProp)
 
 	// featured posts
 	// Pinned posts.
@@ -116,7 +145,7 @@ func (c *Converter) AccountToAS(ctx context.Context, a *gtsmodel.Account) (vocab
 	}
 	featuredProp := streams.NewTootFeaturedProperty()
 	featuredProp.SetIRI(featuredURI)
-	person.SetTootFeatured(featuredProp)
+	accountable.SetTootFeatured(featuredProp)
 
 	// featuredTags
 	// NOT IMPLEMENTED
@@ -125,7 +154,7 @@ func (c *Converter) AccountToAS(ctx context.Context, a *gtsmodel.Account) (vocab
 	// Used for Webfinger lookup. Must be unique on the domain, and must correspond to a Webfinger acct: URI.
 	preferredUsernameProp := streams.NewActivityStreamsPreferredUsernameProperty()
 	preferredUsernameProp.SetXMLSchemaString(a.Username)
-	person.SetActivityStreamsPreferredUsername(preferredUsernameProp)
+	accountable.SetActivityStreamsPreferredUsername(preferredUsernameProp)
 
 	// name
 	// Used as profile display name.
@@ -135,14 +164,14 @@ func (c *Converter) AccountToAS(ctx context.Context, a *gtsmodel.Account) (vocab
 	} else {
 		nameProp.AppendXMLSchemaString(a.Username)
 	}
-	person.SetActivityStreamsName(nameProp)
+	accountable.SetActivityStreamsName(nameProp)
 
 	// summary
 	// Used as profile bio.
 	if a.Note != "" {
 		summaryProp := streams.NewActivityStreamsSummaryProperty()
 		summaryProp.AppendXMLSchemaString(a.Note)
-		person.SetActivityStreamsSummary(summaryProp)
+		accountable.SetActivityStreamsSummary(summaryProp)
 	}
 
 	// url
@@ -153,19 +182,19 @@ func (c *Converter) AccountToAS(ctx context.Context, a *gtsmodel.Account) (vocab
 	}
 	urlProp := streams.NewActivityStreamsUrlProperty()
 	urlProp.AppendIRI(profileURL)
-	person.SetActivityStreamsUrl(urlProp)
+	accountable.SetActivityStreamsUrl(urlProp)
 
 	// manuallyApprovesFollowers
 	// Will be shown as a locked account.
 	manuallyApprovesFollowersProp := streams.NewActivityStreamsManuallyApprovesFollowersProperty()
 	manuallyApprovesFollowersProp.Set(*a.Locked)
-	person.SetActivityStreamsManuallyApprovesFollowers(manuallyApprovesFollowersProp)
+	accountable.SetActivityStreamsManuallyApprovesFollowers(manuallyApprovesFollowersProp)
 
 	// discoverable
 	// Will be shown in the profile directory.
 	discoverableProp := streams.NewTootDiscoverableProperty()
 	discoverableProp.Set(*a.Discoverable)
-	person.SetTootDiscoverable(discoverableProp)
+	accountable.SetTootDiscoverable(discoverableProp)
 
 	// devices
 	// NOT IMPLEMENTED, probably won't implement
@@ -183,7 +212,7 @@ func (c *Converter) AccountToAS(ctx context.Context, a *gtsmodel.Account) (vocab
 			alsoKnownAsURIs[i] = uri
 		}
 
-		ap.SetAlsoKnownAs(person, alsoKnownAsURIs)
+		ap.SetAlsoKnownAs(accountable, alsoKnownAsURIs)
 	}
 
 	// movedTo
@@ -194,7 +223,7 @@ func (c *Converter) AccountToAS(ctx context.Context, a *gtsmodel.Account) (vocab
 			return nil, err
 		}
 
-		ap.SetMovedTo(person, movedTo)
+		ap.SetMovedTo(accountable, movedTo)
 	}
 
 	// publicKey
@@ -235,7 +264,7 @@ func (c *Converter) AccountToAS(ctx context.Context, a *gtsmodel.Account) (vocab
 	publicKeyProp.AppendW3IDSecurityV1PublicKey(publicKey)
 
 	// set the public key property on the Person
-	person.SetW3IDSecurityV1PublicKey(publicKeyProp)
+	accountable.SetW3IDSecurityV1PublicKey(publicKeyProp)
 
 	// tags
 	tagProp := streams.NewActivityStreamsTagProperty()
@@ -263,7 +292,7 @@ func (c *Converter) AccountToAS(ctx context.Context, a *gtsmodel.Account) (vocab
 	// tag -- hashtags
 	// TODO
 
-	person.SetActivityStreamsTag(tagProp)
+	accountable.SetActivityStreamsTag(tagProp)
 
 	// attachment
 	// Used for profile fields.
@@ -284,7 +313,7 @@ func (c *Converter) AccountToAS(ctx context.Context, a *gtsmodel.Account) (vocab
 			attachmentProp.AppendSchemaPropertyValue(propertyValue)
 		}
 
-		person.SetActivityStreamsAttachment(attachmentProp)
+		accountable.SetActivityStreamsAttachment(attachmentProp)
 	}
 
 	// endpoints
@@ -320,7 +349,7 @@ func (c *Converter) AccountToAS(ctx context.Context, a *gtsmodel.Account) (vocab
 			iconImage.SetActivityStreamsUrl(avatarURLProperty)
 
 			iconProperty.AppendActivityStreamsImage(iconImage)
-			person.SetActivityStreamsIcon(iconProperty)
+			accountable.SetActivityStreamsIcon(iconProperty)
 		}
 	}
 
@@ -354,20 +383,27 @@ func (c *Converter) AccountToAS(ctx context.Context, a *gtsmodel.Account) (vocab
 			headerImage.SetActivityStreamsUrl(headerURLProperty)
 
 			headerProperty.AppendActivityStreamsImage(headerImage)
-			person.SetActivityStreamsImage(headerProperty)
+			accountable.SetActivityStreamsImage(headerProperty)
 		}
 	}
 
-	return person, nil
+	return accountable, nil
 }
 
-// AccountToASMinimal converts a gts model account into an activity streams person, suitable for federation.
+// AccountToASMinimal converts a gts model account
+// into an activity streams person or service.
 //
-// The returned account will just have the Type, Username, PublicKey, and ID properties set. This is
-// suitable for serving to requesters to whom we want to give as little information as possible because
-// we don't trust them (yet).
-func (c *Converter) AccountToASMinimal(ctx context.Context, a *gtsmodel.Account) (vocab.ActivityStreamsPerson, error) {
-	person := streams.NewActivityStreamsPerson()
+// The returned account will just have the Type, Username,
+// PublicKey, and ID properties set. This is suitable for
+// serving to requesters to whom we want to give as little
+// information as possible because we don't trust them (yet).
+func (c *Converter) AccountToASMinimal(
+	ctx context.Context,
+	a *gtsmodel.Account,
+) (ap.Accountable, error) {
+	// Use appropriate underlying
+	// actor type of accountable.
+	accountable := accountableForActorType(a.ActorType)
 
 	// id should be the activitypub URI of this user
 	// something like https://example.org/users/example_user
@@ -377,13 +413,13 @@ func (c *Converter) AccountToASMinimal(ctx context.Context, a *gtsmodel.Account)
 	}
 	idProp := streams.NewJSONLDIdProperty()
 	idProp.SetIRI(profileIDURI)
-	person.SetJSONLDId(idProp)
+	accountable.SetJSONLDId(idProp)
 
 	// preferredUsername
 	// Used for Webfinger lookup. Must be unique on the domain, and must correspond to a Webfinger acct: URI.
 	preferredUsernameProp := streams.NewActivityStreamsPreferredUsernameProperty()
 	preferredUsernameProp.SetXMLSchemaString(a.Username)
-	person.SetActivityStreamsPreferredUsername(preferredUsernameProp)
+	accountable.SetActivityStreamsPreferredUsername(preferredUsernameProp)
 
 	// publicKey
 	// Required for signatures.
@@ -423,9 +459,9 @@ func (c *Converter) AccountToASMinimal(ctx context.Context, a *gtsmodel.Account)
 	publicKeyProp.AppendW3IDSecurityV1PublicKey(publicKey)
 
 	// set the public key property on the Person
-	person.SetW3IDSecurityV1PublicKey(publicKeyProp)
+	accountable.SetW3IDSecurityV1PublicKey(publicKeyProp)
 
-	return person, nil
+	return accountable, nil
 }
 
 // StatusToAS converts a gts model status into an ActivityStreams Statusable implementation, suitable for federation
@@ -444,7 +480,7 @@ func (c *Converter) StatusToAS(ctx context.Context, s *gtsmodel.Status) (ap.Stat
 		poll := streams.NewActivityStreamsQuestion()
 
 		// Add required status poll data to AS Question.
-		if err := c.addPollToAS(ctx, s.Poll, poll); err != nil {
+		if err := c.addPollToAS(s.Poll, poll); err != nil {
 			return nil, gtserror.Newf("error converting poll: %w", err)
 		}
 
@@ -484,10 +520,11 @@ func (c *Converter) StatusToAS(ctx context.Context, s *gtsmodel.Status) (ap.Stat
 		status.SetActivityStreamsInReplyTo(inReplyToProp)
 	}
 
-	// published
-	publishedProp := streams.NewActivityStreamsPublishedProperty()
-	publishedProp.Set(s.CreatedAt)
-	status.SetActivityStreamsPublished(publishedProp)
+	// Set created / updated at properties.
+	ap.SetPublished(status, s.CreatedAt)
+	if at := s.EditedAt; !at.IsZero() {
+		ap.SetUpdated(status, at)
+	}
 
 	// url
 	if s.URL != "" {
@@ -674,41 +711,44 @@ func (c *Converter) StatusToAS(ctx context.Context, s *gtsmodel.Status) (ap.Stat
 	status.SetActivityStreamsSensitive(sensitiveProp)
 
 	// interactionPolicy
-	var p *gtsmodel.InteractionPolicy
-	if s.InteractionPolicy != nil {
-		// Use InteractionPolicy
-		// set on the status.
-		p = s.InteractionPolicy
-	} else {
-		// Fall back to default policy
-		// for the status's visibility.
-		p = gtsmodel.DefaultInteractionPolicyFor(s.Visibility)
-	}
-	policy, err := c.InteractionPolicyToASInteractionPolicy(ctx, p, s)
-	if err != nil {
-		return nil, fmt.Errorf("error creating interactionPolicy: %w", err)
-	}
-
-	policyProp := streams.NewGoToSocialInteractionPolicyProperty()
-	policyProp.AppendGoToSocialInteractionPolicy(policy)
-	status.SetGoToSocialInteractionPolicy(policyProp)
-
-	// Parse + set approvedBy.
-	if s.ApprovedByURI != "" {
-		approvedBy, err := url.Parse(s.ApprovedByURI)
+	if ipa, ok := status.(ap.InteractionPolicyAware); ok {
+		var p *gtsmodel.InteractionPolicy
+		if s.InteractionPolicy != nil {
+			// Use InteractionPolicy
+			// set on the status.
+			p = s.InteractionPolicy
+		} else {
+			// Fall back to default policy
+			// for the status's visibility.
+			p = gtsmodel.DefaultInteractionPolicyFor(s.Visibility)
+		}
+		policy, err := c.InteractionPolicyToASInteractionPolicy(ctx, p, s)
 		if err != nil {
-			return nil, fmt.Errorf("error parsing approvedBy: %w", err)
+			return nil, fmt.Errorf("error creating interactionPolicy: %w", err)
 		}
 
-		approvedByProp := streams.NewGoToSocialApprovedByProperty()
-		approvedByProp.Set(approvedBy)
-		status.SetGoToSocialApprovedBy(approvedByProp)
+		// Set interaction policy.
+		policyProp := streams.NewGoToSocialInteractionPolicyProperty()
+		policyProp.AppendGoToSocialInteractionPolicy(policy)
+		ipa.SetGoToSocialInteractionPolicy(policyProp)
+
+		// Parse + set approvedBy.
+		if s.ApprovedByURI != "" {
+			approvedBy, err := url.Parse(s.ApprovedByURI)
+			if err != nil {
+				return nil, fmt.Errorf("error parsing approvedBy: %w", err)
+			}
+
+			approvedByProp := streams.NewGoToSocialApprovedByProperty()
+			approvedByProp.Set(approvedBy)
+			ipa.SetGoToSocialApprovedBy(approvedByProp)
+		}
 	}
 
 	return status, nil
 }
 
-func (c *Converter) addPollToAS(ctx context.Context, poll *gtsmodel.Poll, dst ap.Pollable) error {
+func (c *Converter) addPollToAS(poll *gtsmodel.Poll, dst ap.Pollable) error {
 	var optionsProp interface {
 		// the minimum interface for appending AS Notes
 		// to an AS type options property of some kind.
@@ -1701,10 +1741,14 @@ func (c *Converter) ReportToASFlag(ctx context.Context, r *gtsmodel.Report) (voc
 // PollVoteToASCreate converts a vote on a poll into a Create
 // activity, suitable for federation, with each choice in the
 // vote appended as a Note to the Create's Object field.
-func (c *Converter) PollVoteToASCreate(
+//
+// TODO: as soon as other AP server implementations support
+// the use of multiple objects in a single create, update this
+// to return just the one create event again.
+func (c *Converter) PollVoteToASCreates(
 	ctx context.Context,
 	vote *gtsmodel.PollVote,
-) (vocab.ActivityStreamsCreate, error) {
+) ([]vocab.ActivityStreamsCreate, error) {
 	if len(vote.Choices) == 0 {
 		panic("no vote.Choices")
 	}
@@ -1743,22 +1787,25 @@ func (c *Converter) PollVoteToASCreate(
 		return nil, gtserror.Newf("invalid account uri: %w", err)
 	}
 
-	// Allocate Create activity and address 'To' poll author.
-	create := streams.NewActivityStreamsCreate()
-	ap.AppendTo(create, pollAuthorIRI)
+	// Parse each choice to a Note and add it to the list of Creates.
+	creates := make([]vocab.ActivityStreamsCreate, len(vote.Choices))
+	for i, choice := range vote.Choices {
 
-	// Create ID formatted as: {$voterIRI}/activity#vote/{$statusIRI}.
-	id := author.URI + "/activity#vote/" + poll.Status.URI
-	ap.MustSet(ap.SetJSONLDIdStr, ap.WithJSONLDId(create), id)
+		// Allocate Create activity and address 'To' poll author.
+		create := streams.NewActivityStreamsCreate()
+		ap.AppendTo(create, pollAuthorIRI)
 
-	// Set Create actor appropriately.
-	ap.AppendActorIRIs(create, authorIRI)
+		// Create ID formatted as: {$voterIRI}/activity#vote{$index}/{$statusIRI}.
+		createID := fmt.Sprintf("%s/activity#vote%d/%s", author.URI, i, poll.Status.URI)
+		ap.MustSet(ap.SetJSONLDIdStr, ap.WithJSONLDId(create), createID)
 
-	// Set publish time for activity.
-	ap.SetPublished(create, vote.CreatedAt)
+		// Set Create actor appropriately.
+		ap.AppendActorIRIs(create, authorIRI)
 
-	// Parse each choice to a Note and add it to the Create.
-	for _, choice := range vote.Choices {
+		// Set publish time for activity.
+		ap.SetPublished(create, vote.CreatedAt)
+
+		// Allocate new note to hold the vote.
 		note := streams.NewActivityStreamsNote()
 
 		// For AP IRI generate from author URI + poll ID + vote choice.
@@ -1775,11 +1822,14 @@ func (c *Converter) PollVoteToASCreate(
 		ap.AppendInReplyTo(note, statusIRI)
 		ap.AppendTo(note, pollAuthorIRI)
 
-		// Append this note as Create Object.
+		// Append this note to the Create Object.
 		appendStatusableToActivity(create, note, false)
+
+		// Set create in slice.
+		creates[i] = create
 	}
 
-	return create, nil
+	return creates, nil
 }
 
 // populateValuesForProp appends the given PolicyValues
@@ -1819,7 +1869,7 @@ func populateValuesForProp[T ap.WithIRI](
 	// Deduplicate the iri strings to
 	// make sure we're not parsing + adding
 	// the same string multiple times.
-	iriStrs = util.Deduplicate(iriStrs)
+	iriStrs = xslices.Deduplicate(iriStrs)
 
 	// Append them to the property.
 	for _, iriStr := range iriStrs {
@@ -1980,7 +2030,19 @@ func (c *Converter) InteractionReqToASAccept(
 
 	objectIRI, err := url.Parse(req.InteractionURI)
 	if err != nil {
-		return nil, gtserror.Newf("invalid target uri: %w", err)
+		return nil, gtserror.Newf("invalid object uri: %w", err)
+	}
+
+	if req.Status == nil {
+		req.Status, err = c.state.DB.GetStatusByID(ctx, req.StatusID)
+		if err != nil {
+			return nil, gtserror.Newf("db error getting interaction req target status: %w", err)
+		}
+	}
+
+	targetIRI, err := url.Parse(req.Status.URI)
+	if err != nil {
+		return nil, gtserror.Newf("invalid interaction req target status uri: %w", err)
 	}
 
 	toIRI, err := url.Parse(req.InteractingAccount.URI)
@@ -1999,6 +2061,10 @@ func (c *Converter) InteractionReqToASAccept(
 	// Object is the interaction URI.
 	ap.AppendObjectIRIs(accept, objectIRI)
 
+	// Target is the URI of the
+	// status being interacted with.
+	ap.AppendTargetIRIs(accept, targetIRI)
+
 	// Address to the owner
 	// of interaction URI.
 	ap.AppendTo(accept, toIRI)
@@ -2060,7 +2126,19 @@ func (c *Converter) InteractionReqToASReject(
 
 	objectIRI, err := url.Parse(req.InteractionURI)
 	if err != nil {
-		return nil, gtserror.Newf("invalid target uri: %w", err)
+		return nil, gtserror.Newf("invalid object uri: %w", err)
+	}
+
+	if req.Status == nil {
+		req.Status, err = c.state.DB.GetStatusByID(ctx, req.StatusID)
+		if err != nil {
+			return nil, gtserror.Newf("db error getting interaction req target status: %w", err)
+		}
+	}
+
+	targetIRI, err := url.Parse(req.Status.URI)
+	if err != nil {
+		return nil, gtserror.Newf("invalid interaction req target status uri: %w", err)
 	}
 
 	toIRI, err := url.Parse(req.InteractingAccount.URI)
@@ -2079,6 +2157,10 @@ func (c *Converter) InteractionReqToASReject(
 	// Object is the interaction URI.
 	ap.AppendObjectIRIs(reject, objectIRI)
 
+	// Target is the URI of the
+	// status being interacted with.
+	ap.AppendTargetIRIs(reject, targetIRI)
+
 	// Address to the owner
 	// of interaction URI.
 	ap.AppendTo(reject, toIRI)
